from optparse import OptionParser
from WMCore.Database.DBFactory import DBFactory
from WMCore.Database.Transaction import Transaction
import logging

def input():
    parser = OptionParser()
    parser.add_option("-d", "--dburl", dest="dburl", 
              help="Database to connect to and use: oracle://user:password@tns")
    return parser.parse_args()

def addGroupsAndSites(db):
    map = {
    'Forward' : ['T2_US_Wisconsin', 'T2_DE_DESY'],
    'QCD' : ['T2_DE_DESY', 'T2_FR_CCIN2P3', 'T2_US_Caltech', 'T2_US_MIT'],
    'Higgs' : ['T2_ES_IFCA', 'T2_FR_GRIF_LLR', 'T2_IT_Rome', 'T2_US_Caltech', 'T2_US_MIT'],
    'EWK' : ['T2_ES_CIEMAT', 'T2_FR_CCIN2P3', 'T2_IT_Legnaro', 'T2_UK_London_Brunel', 'T2_US_UCSD'],
    'SUSY' : ['T2_AT_Vienna', 'T2_DE_RWTH', 'T2_IT_Bari', 'T2_UK_London_IC', 'T2_US_Florida'],
    'Top' : ['T2_DE_DESY', 'T2_ES_IFCA', 'T2_FR_IPHC', 'T2_US_UCSD'],
    'Exotica' : ['T2_FR_GRIF_LLR', 'T2_RU', 'T2_UK_SGrid_RALPP', 'T2_US_Purdue'],
    'B-Physics' : ['T2_CH_CSCS', 'T2_CN_Beijing', 'T2_FI_HIP', 'T2_US_MIT'],
    'Heavy Ion' : ['T2_RU'],
    'E-gamma' : ['T2_FR_GRIF_IRFU', 'T2_IT_Rome', 'T2_UK_London_IC', 'T2_US_Caltech', 'T2_US_UCSD'],
    'Jets/MET HCAL' : ['T2_DE_DESY', 'T2_FI_HIP', 'T2_KR_KNU', 'T2_US_Purdue', 'T2_RU'],
    'Muon' : ['T2_ES_CIEMAT', 'T2_IT_Legnaro', 'T2_RU', 'T2_US_Purdue', 'T2_US_Florida'],
    'B-Tagging' : ['T2_AT_Vienna', 'T2_BR_UERJ', 'T2_FR_IPHC', 'T2_US_Nebraska'],
    'Tracker' : ['T2_DE_RWTH', 'T2_FR_CCIN2P3', 'T2_IT_Pisa', 'T2_US_Nebraska'],
    'Tau/PFlow' : ['T2_EE_Estonia', 'T2_FR_CCIN2P3', 'T2_IT_Pisa', 'T2_US_Florida'],
    'Trigger' : ['T2_ES_CIEMAT', 'T2_UK_London_IC', 'T2_US_Wisconsin']
    }
    
    
    groupsql = "insert into user_group (id, name) values (user_group_sq.nextval, :usergroup)"
    sitesql = """insert into site_group_association (site, user_group) values (
            (select site_cms_name_map.site_id from site_cms_name_map where cms_name_id = 
                (
                    select id from cms_name where cms_name.name = :site
                )
            ),
            (select user_group.id from user_group where name = :usergroup)
        )"""
    
    
    for group in map.keys():
        trans = Transaction(db)
        # Add the group, may already exist, so need a try/except
        try:
            trans.processData(groupsql, {'usergroup' :group})
        except Exception, e:
            print "group exists, or some other problem"
            print e
        failcount = 0
        for site in map[group]:
            # Add the site/group association, may already exist, so need a try/except
            print site
            try:
                trans.processData(sitesql, {'usergroup' :group, 'site' :site})
            except Exception, e:
                print "site/group association exists, or some other problem"
                print e
                failcount += 1
        if failcount < len(map[group]):
            trans.commit()
        else:
            trans.rollback()
                
def main():
    opts, args = input()
    
    db = DBFactory(logging.getLogger('SiteGroupMap'), dburl=opts.dburl).connect()
    
    addGroupsAndSites(db)
    
if __name__ == "__main__":
    main()   