from time import ctime
import os

from Framework import Controller
from Framework import templatepage
from Framework.PluginManager import DeclarePlugin

from cherrypy import expose
from cherrypy import request

from dataprovider.runDataProviders import *
from dataprovider.jobDataProviders import *
from dataprovider.fileDataProviders import *

from guiProvider import *
from utils import *
from consts import *

import globals
import config
import sqlManager

# Table IDs
RUNS_T = 'runsT'
REPACKER_T = 'repackerT'
REPACK_S_T = 'repackST'
MERGE_T = 'mergeT'
MERGE_S_T = 'mergeST'
RECO_T = 'recoT'
RECO_MERGE_T = 'recoMergeT'
RECO_S_T = 'recoST'
RECO_MERGE_S_T = 'recoMergeST'
ALCASKIM_T = 'alcaskimT'
ALCASKIM_MERGE_T = 'alcaskimMergeT'
ALCASKIM_S_T = 'alcaskimST'
ALCASKIM_MERGE_S_T = 'alcaskimMergeST'
RUN_REPACK_T = 'runRepackT'
RUN_CONFIG_KEY_T = 'runConfigKeyT'
RUN_REPACK_MERGE_T = 'runRepackMergeT'
RUN_RECO_T = 'runRecoT'
RUN_RECO_MERGE_T = 'runRecoMergeT'
RUN_ALCASKIM_T = 'runAlcaskimT'
RUN_ALCASKIM_MERGE_T = 'runAlcaskimMergeT'
RUN_STREAMER_T = 'runStreamerT'
RUN_REPACK_STATS_T = 'runRepackStatsT'
RUN_MERGE_STATS_T = 'runMergeStatsT'
RUN_RECO_STATS_T = 'runRecoStatsT'
RUN_RECO_MERGE_STATS_T = 'runRecoMergeStatsT'
RUN_ALCASKIM_STATS_T = 'runAlcaskimStatsT'
RUN_ALCASKIM_MERGE_STATS_T = 'runAlcaskimMergeStatsT'

T_STREAM_FILES = 'streamerFiles'
T_RECO_FILES = 'recoFiles'
T_RECO_JOBS = 'recoJobs'
T_REPACK_FILES = 'repackFiles'
T_REPACK_JOBS = 'repackJobs'
T_MERGE_FILES = 'mergeFiles'
T_MERGE_JOBS = 'mergeJobs'
T_RECO_MERGE_JOBS = 'recoMergeJobs'
T_RECO_MERGE_FILES = 'recoMergeFiles'
T_ALCA_JOBS = 'alcaJobs'
T_ALCA_FILES = 'alcaFiles'
T_ALCA_MERGE_JOBS = 'alcaMergeJobs'
T_ALCA_MERGE_FILES = 'alcaMergeFiles'

T_JOBS = 'jobs'

# Identify cheetah template full path
TMPL_PATH = os.path.join(__file__.rsplit ("/", 1)[0], "Templates/")

class T0Mon (Controller):
    """
    Web page controller
    Provides GUI for user and communicates with backend
    User input should be dealt with only in this class
    """

    def __init__(self, context):
        """
        Init the controller
        """
        Controller.__init__(self, context, __file__)
        # get logger from context
        globals.logger = self.context.Logger()
        # connect to DB
        sqlManager.initBasicConnection()

    #### Public ####

    @expose
    def repackJob(self, id, **kwds):
        """
        Display repack job history
        """
        globals.pageParams = kwds
        entity = 'Repacker job'

        try:
            id = int(id)
        except ValueError:
            return self.templatePage (
                "error", {
                    'msg': "Bad " + entity + " id",
                    "tmplPath" : TMPL_PATH
                }
            )

        try:
            params = self._getTableParams(T_REPACK_JOBS)
            repackJobData = RepackJobByIdsDP([id], params.orderby, params.asc, params.page)
            repackJobs = getTableGui(T_REPACK_JOBS, repackJobData)
            repackJobs = wrapInBorder(repackJobs)
        except: repackJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_STREAM_FILES)
            repackJobIds = [int(line[0]) for line in repackJobData.data]
            streamerData = StreamerByRepackJobIdsDP(repackJobIds, params.orderby, params.asc, params.page)
            streamerFiles = getTableGui(T_STREAM_FILES, streamerData)
        except: streamerFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_REPACK_FILES)
	    streamerIds = [int(line[0]) for line in streamerData.data]
            repackData = RepackByStreamerIdsDP(streamerIds, params.orderby, params.asc, params.page)
            repackFiles = getTableGui(T_REPACK_FILES, repackData)
        except: repackFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_MERGE_JOBS)
            repackIds = [int(line[0]) for line in repackData.data]
            mergeData = RepackMergeJobByRepackIdsDP(repackIds, params.orderby, params.asc, params.page)
            mergeJobs = getTableGui(T_MERGE_JOBS, mergeData)
        except: mergeJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_MERGE_FILES)
            mergeData = RepackMergeByRepackIdsDP(repackIds, params.orderby, params.asc, params.page)
            mergeFiles = getTableGui(T_MERGE_FILES, mergeData)
        except: mergeFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_JOBS)
            mergeIds = [int(line[0]) for line in mergeData.data]
            recoData = RecoJobByMergeIdsDP(mergeIds, params.orderby, params.asc, params.page)
            recoJobs = getTableGui(T_RECO_JOBS, recoData)
        except:
            recoJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_FILES)
            recoData = RecoByMergeIdsDP(mergeIds, params.orderby, params.asc, params.page)
            recoFiles = getTableGui(T_RECO_FILES, recoData)
        except:
            recoFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_MERGE_JOBS)
            recoIds = [int(line[0]) for line in recoData.data]
            recoMergeData = RecoMergeJobByRecoIdsDP(recoIds, params.orderby, params.asc, params.page)
            recoMergeJobs = getTableGui(T_RECO_MERGE_JOBS, recoMergeData)
        except:
            recoMergeJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_MERGE_FILES)
            recoMergeData = RecoMergeByRecoIdsDP(recoIds, params.orderby, params.asc, params.page)
            recoMergeFiles = getTableGui(T_RECO_MERGE_FILES, recoMergeData)
        except:
            recoMergeFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_ALCA_JOBS)
            recoMergeIds = [int(line[0]) for line in recoMergeData.data]
            alcaSkimData = AlcaSkimJobByRecoMergeIdsDP(recoMergeIds, params.orderby, params.asc, params.page)
            alcaJobs = getTableGui(T_ALCA_JOBS, alcaSkimData)
        except:
            alcaJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_ALCA_FILES)
            alcaSkimData = AlcaSkimByRecoMergeIdsDp(recoMergeIds, params.orderby, params.asc, params.page)
            alcaFiles = getTableGui(T_ALCA_FILES, alcaSkimData)
        except:
            alcaFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_ALCA_MERGE_JOBS)
            alcaSkimIds = [int(line[0]) for line in alcaSkimData.data]
            alcaSkimMergeData = AlcaSkimMergeJobByAlcaSkimIdsDP(alcaSkimIds, params.orderby, params.asc, params.page)
            alcaMergeJobs = getTableGui(T_ALCA_MERGE_FILES, alcaSkimMergeData)
        except:
            alcaMergeJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_ALCA_MERGE_FILES)
            alcaSkimMergeData = AlcaSkimMergeByAlcaSkimIdsDP(alcaSkimIds, params.orderby, params.asc, params.page)
            alcaMergeFiles = getTableGui(T_ALCA_MERGE_FILES, alcaSkimMergeData)
        except:
            alcaMergeFiles = formatExceptionInfo()

        return self.templatePage (
                "history", {
                    T_STREAM_FILES: streamerFiles,
                    T_REPACK_JOBS : repackJobs,
                    T_REPACK_FILES : repackFiles,
                    T_MERGE_JOBS : mergeJobs,
                    T_MERGE_FILES : mergeFiles,
                    T_RECO_JOBS : recoJobs,
                    T_RECO_FILES : recoFiles,
                    T_RECO_MERGE_JOBS : recoMergeJobs,
                    T_RECO_MERGE_FILES : recoMergeFiles,
                    T_ALCA_JOBS : alcaJobs,
                    T_ALCA_FILES : alcaFiles,
                    T_ALCA_MERGE_JOBS : alcaMergeJobs,
                    T_ALCA_MERGE_FILES : alcaMergeFiles,
                    'entity' : entity + ' ' + str(id),
                    "tmplPath" : TMPL_PATH
                }
            )


    @expose
    def recoMergeJob(self, id, **kwds):
        """
        Display reco merge job history
        """
        globals.pageParams = kwds
        entity = 'Reco Merge job'

        try:
            id = int(id)
        except ValueError:
            return self.templatePage (
                "error", {
                    'msg': "Bad " + entity + " id",
                    "tmplPath" : TMPL_PATH
                }
            )

        try:
            params = self._getTableParams(T_RECO_MERGE_JOBS)
            recoMergeData = RecoMergeJobByIdsDP([id], params.orderby, params.asc, params.page)
            recoMergeJobs = getTableGui(T_RECO_MERGE_JOBS, recoMergeData)
            recoMergeJobs = wrapInBorder(recoMergeJobs)
        except:
            recoMergeJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_FILES)
            recoMergeJobIds = [int(line[0]) for line in recoMergeData.data]
            recoData = RecoByRecoMergeJobIdsDP(recoMergeJobIds, params.orderby, params.asc, params.page)
            recoFiles = getTableGui(T_RECO_FILES, recoData)
        except:
            recoFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_MERGE_FILES)
            recoIds = [int(line[0]) for line in recoData.data]
            recoMergeData = RecoMergeByRecoIdsDP(recoIds, params.orderby, params.asc, params.page)
            recoMergeFiles = getTableGui(T_RECO_MERGE_FILES, recoMergeData)
        except:
            recoMergeFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_ALCA_JOBS)
            recoMergeIds = [int(line[0]) for line in recoMergeData.data]
            alcaSkimData = AlcaSkimJobByRecoMergeIdsDP(recoMergeIds, params.orderby, params.asc, params.page)
            alcaJobs = getTableGui(T_ALCA_JOBS, alcaSkimData)
        except:
            alcaJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_ALCA_FILES)
            alcaSkimData = AlcaSkimByRecoMergeIdsDp(recoMergeIds, params.orderby, params.asc, params.page)
            alcaFiles = getTableGui(T_ALCA_FILES, alcaSkimData)
        except:
            alcaFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_ALCA_MERGE_JOBS)
            alcaSkimIds = [int(line[0]) for line in alcaSkimData.data]
            alcaSkimMergeData = AlcaSkimMergeJobByAlcaSkimIdsDP(alcaSkimIds, params.orderby, params.asc, params.page)
            alcaMergeJobs = getTableGui(T_ALCA_MERGE_FILES, alcaSkimMergeData)
        except:
            alcaMergeJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_ALCA_MERGE_FILES)
            alcaSkimMergeData = AlcaSkimMergeByAlcaSkimIdsDP(alcaSkimIds, params.orderby, params.asc, params.page)
            alcaMergeFiles = getTableGui(T_ALCA_MERGE_FILES, alcaSkimMergeData)
        except:
            alcaMergeFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_JOBS)
            recoJobData = RecoJobByRecoIdsDP(recoIds, params.orderby, params.asc, params.page)
            recoJobs = getTableGui(T_RECO_JOBS, recoJobData)
        except:
            recoJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_MERGE_FILES)
            recoJobIds = [int(line[0]) for line in recoJobData.data]
            mergeFilesData = RepackMergeByRecoJobIdsDP(recoJobIds, params.orderby, params.asc, params.page)
            mergeFiles = getTableGui(T_MERGE_FILES, mergeFilesData)
        except: mergeFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_MERGE_JOBS)
            mergeIds = [int(line[0]) for line in mergeFilesData.data]
            mergeData = RepackMergeJobByRepackMergeIdsDP(mergeIds, params.orderby, params.asc, params.page)
            mergeJobs = getTableGui(T_MERGE_JOBS, mergeData)
        except: mergeJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_REPACK_FILES)
            repackData = RepackByRepackMergeIdsDP(mergeIds, params.orderby, params.asc, params.page)
            repackFiles = getTableGui(T_REPACK_FILES, repackData)
        except: repackFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_REPACK_JOBS)
            repackIds = [int(line[0]) for line in repackData.data]
            repackData = RepackJobByRepackIdsDP(repackIds, params.orderby, params.asc, params.page)
            repackJobs = getTableGui(T_REPACK_JOBS, repackData)
        except: repackJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_STREAM_FILES)
            repackJobIds = [int(line[0]) for line in repackData.data]
            streamerData = StreamerByRepackJobIdsDP(repackJobIds, params.orderby, params.asc, params.page)
            streamerFiles = getTableGui(T_STREAM_FILES, streamerData)
        except: streamerFiles = formatExceptionInfo()

        return self.templatePage (
                "history", {
                    T_STREAM_FILES: streamerFiles,
                    T_REPACK_JOBS : repackJobs,
                    T_REPACK_FILES : repackFiles,
                    T_MERGE_JOBS : mergeJobs,
                    T_MERGE_FILES : mergeFiles,
                    T_RECO_JOBS : recoJobs,
                    T_RECO_FILES : recoFiles,
                    T_RECO_MERGE_JOBS : recoMergeJobs,
                    T_RECO_MERGE_FILES : recoMergeFiles,
                    T_ALCA_JOBS : alcaJobs,
                    T_ALCA_FILES : alcaFiles,
                    T_ALCA_MERGE_JOBS : alcaMergeJobs,
                    T_ALCA_MERGE_FILES : alcaMergeFiles,
                    'entity' : entity + ' ' + str(id),
                    "tmplPath" : TMPL_PATH
                }
            )


    @expose
    def alcaSkimJob(self, id, **kwds):
        """
        Display alcaSkim job history
        """
        globals.pageParams = kwds
        entity = 'AlcaSkim job'

        try:
            id = int(id)
        except ValueError:
            return self.templatePage (
                "error", {
                    'msg': "Bad " + entity + " id",
                    "tmplPath" : TMPL_PATH
                }
            )

        try:
            params = self._getTableParams(T_ALCA_JOBS)
            alcaSkimData = AlcaSkimJobByIdsDP([id], params.orderby, params.asc, params.page)
            alcaJobs = getTableGui(T_ALCA_JOBS, alcaSkimData)
	    alcaJobs = wrapInBorder(alcaJobs)
        except:
            alcaJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_MERGE_FILES)
            alcaSkimJobIds = [int(line[0]) for line in alcaSkimData.data]
            recoMergeData = RecoMergeByAlcaSkimJobIdsDP(alcaSkimJobIds, params.orderby, params.asc, params.page)
            recoMergeFiles = getTableGui(T_RECO_MERGE_FILES, recoMergeData)
        except:
            recoMergeFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_ALCA_FILES)
            recoMergeIds = [int(line[0]) for line in recoMergeData.data]
            alcaSkimData = AlcaSkimByRecoMergeIdsDp(recoMergeIds, params.orderby, params.asc, params.page)
            alcaFiles = getTableGui(T_ALCA_FILES, alcaSkimData)
        except:
            alcaFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_ALCA_MERGE_JOBS)
            alcaSkimIds = [int(line[0]) for line in alcaSkimData.data]
            alcaSkimMergeData = AlcaSkimMergeJobByAlcaSkimIdsDP(alcaSkimIds, params.orderby, params.asc, params.page)
            alcaMergeJobs = getTableGui(T_ALCA_MERGE_FILES, alcaSkimMergeData)
        except:
            alcaMergeJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_ALCA_MERGE_FILES)
            alcaSkimMergeData = AlcaSkimMergeByAlcaSkimIdsDP(alcaSkimIds, params.orderby, params.asc, params.page)
            alcaMergeFiles = getTableGui(T_ALCA_MERGE_FILES, alcaSkimMergeData)
        except:
            alcaMergeFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_MERGE_JOBS)
            recoMergeData = RecoMergeJobByRecoMergeIdsDP(recoMergeIds, params.orderby, params.asc, params.page)
            recoMergeJobs = getTableGui(T_RECO_MERGE_JOBS, recoMergeData)
        except:
            recoMergeJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_FILES)
            recoMergeJobIds = [int(line[0]) for line in recoMergeData.data]
            recoData = RecoByRecoMergeJobIdsDP(recoMergeJobIds, params.orderby, params.asc, params.page)
            recoFiles = getTableGui(T_RECO_FILES, recoData)
        except:
            recoFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_JOBS)
            recoIds = [int(line[0]) for line in recoData.data]
            recoJobData = RecoJobByRecoIdsDP(recoIds, params.orderby, params.asc, params.page)
            recoJobs = getTableGui(T_RECO_JOBS, recoJobData)
        except:
            recoJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_MERGE_FILES)
            recoJobIds = [int(line[0]) for line in recoJobData.data]
            mergeFilesData = RepackMergeByRecoJobIdsDP(recoJobIds, params.orderby, params.asc, params.page)
            mergeFiles = getTableGui(T_MERGE_FILES, mergeFilesData)
        except: mergeFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_MERGE_JOBS)
            mergeIds = [int(line[0]) for line in mergeFilesData.data]
            mergeData = RepackMergeJobByRepackMergeIdsDP(mergeIds, params.orderby, params.asc, params.page)
            mergeJobs = getTableGui(T_MERGE_JOBS, mergeData)
        except: mergeJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_REPACK_FILES)
            repackData = RepackByRepackMergeIdsDP(mergeIds, params.orderby, params.asc, params.page)
            repackFiles = getTableGui(T_REPACK_FILES, repackData)
        except: repackFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_REPACK_JOBS)
            repackIds = [int(line[0]) for line in repackData.data]
            repackData = RepackJobByRepackIdsDP(repackIds, params.orderby, params.asc, params.page)
            repackJobs = getTableGui(T_REPACK_JOBS, repackData)
        except: repackJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_STREAM_FILES)
            repackJobIds = [int(line[0]) for line in repackData.data]
            streamerData = StreamerByRepackJobIdsDP(repackJobIds, params.orderby, params.asc, params.page)
            streamerFiles = getTableGui(T_STREAM_FILES, streamerData)
        except: streamerFiles = formatExceptionInfo()

        return self.templatePage (
                "history", {
                    T_STREAM_FILES: streamerFiles,
                    T_REPACK_JOBS : repackJobs,
                    T_REPACK_FILES : repackFiles,
                    T_MERGE_JOBS : mergeJobs,
                    T_MERGE_FILES : mergeFiles,
                    T_RECO_JOBS : recoJobs,
                    T_RECO_FILES : recoFiles,
                    T_RECO_MERGE_JOBS : recoMergeJobs,
                    T_RECO_MERGE_FILES : recoMergeFiles,
                    T_ALCA_JOBS : alcaJobs,
                    T_ALCA_FILES : alcaFiles,
                    T_ALCA_MERGE_JOBS : alcaMergeJobs,
                    T_ALCA_MERGE_FILES : alcaMergeFiles,
                    'entity' : entity + ' ' + str(id),
                    "tmplPath" : TMPL_PATH
                }
            )


    @expose
    def alcaSkimMergeJob(self, id, **kwds):
        """
        Display alcaSkim merge job history
        """
        globals.pageParams = kwds
        entity = 'AlcaSkim Merge job'

        try:
            id = int(id)
        except ValueError:
            return self.templatePage (
                "error", {
                    'msg': "Bad " + entity + " id",
                    "tmplPath" : TMPL_PATH
                }
            )

        try:
            params = self._getTableParams(T_ALCA_MERGE_JOBS)
            alcaSkimMergeData = AlcaSkimMergeJobByIdsDP([id], params.orderby, params.asc, params.page)
            alcaMergeJobs = getTableGui(T_ALCA_MERGE_FILES, alcaSkimMergeData)
            alcaMergeJobs = wrapInBorder(alcaMergeJobs)
        except:
            alcaMergeJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_ALCA_FILES)
	    alcaSkimMergeJobIds = [int(line[0]) for line in alcaSkimMergeData.data]
            alcaSkimData = AlcaSkimByAlcaSkimMergeJobIdsDP(alcaSkimMergeJobIds, params.orderby, params.asc, params.page)
            alcaFiles = getTableGui(T_ALCA_FILES, alcaSkimData)
        except:
            alcaFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_ALCA_MERGE_FILES)
            alcaSkimIds = [int(line[0]) for line in alcaSkimData.data]
            alcaSkimMergeData = AlcaSkimMergeByAlcaSkimIdsDP(alcaSkimIds, params.orderby, params.asc, params.page)
            alcaMergeFiles = getTableGui(T_ALCA_MERGE_FILES, alcaSkimMergeData)
        except:
            alcaMergeFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_ALCA_JOBS)
            alcaSkimData = AlcaSkimJobByAlcaSkimIdsDP(alcaSkimIds, params.orderby, params.asc, params.page)
            alcaJobs = getTableGui(T_ALCA_JOBS, alcaSkimData)
        except:
            alcaJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_MERGE_FILES)
            alcaSkimJobIds = [int(line[0]) for line in alcaSkimData.data]
            recoMergeData = RecoMergeByAlcaSkimJobIdsDP(alcaSkimJobIds, params.orderby, params.asc, params.page)
            recoMergeFiles = getTableGui(T_RECO_MERGE_FILES, recoMergeData)
        except:
            recoMergeFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_MERGE_JOBS)
            recoMergeIds = [int(line[0]) for line in recoMergeData.data]
            recoMergeData = RecoMergeJobByRecoMergeIdsDP(recoMergeIds, params.orderby, params.asc, params.page)
            recoMergeJobs = getTableGui(T_RECO_MERGE_JOBS, recoMergeData)
        except:
            recoMergeJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_FILES)
            recoMergeJobIds = [int(line[0]) for line in recoMergeData.data]
            recoData = RecoByRecoMergeJobIdsDP(recoMergeJobIds, params.orderby, params.asc, params.page)
            recoFiles = getTableGui(T_RECO_FILES, recoData)
        except:
            recoFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_JOBS)
            recoIds = [int(line[0]) for line in recoData.data]
            recoJobData = RecoJobByRecoIdsDP(recoIds, params.orderby, params.asc, params.page)
            recoJobs = getTableGui(T_RECO_JOBS, recoJobData)
        except:
            recoJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_MERGE_FILES)
            recoJobIds = [int(line[0]) for line in recoJobData.data]
            mergeFilesData = RepackMergeByRecoJobIdsDP(recoJobIds, params.orderby, params.asc, params.page)
            mergeFiles = getTableGui(T_MERGE_FILES, mergeFilesData)
        except: mergeFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_MERGE_JOBS)
            mergeIds = [int(line[0]) for line in mergeFilesData.data]
            mergeData = RepackMergeJobByRepackMergeIdsDP(mergeIds, params.orderby, params.asc, params.page)
            mergeJobs = getTableGui(T_MERGE_JOBS, mergeData)
        except: mergeJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_REPACK_FILES)
            repackData = RepackByRepackMergeIdsDP(mergeIds, params.orderby, params.asc, params.page)
            repackFiles = getTableGui(T_REPACK_FILES, repackData)
        except: repackFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_REPACK_JOBS)
            repackIds = [int(line[0]) for line in repackData.data]
            repackData = RepackJobByRepackIdsDP(repackIds, params.orderby, params.asc, params.page)
            repackJobs = getTableGui(T_REPACK_JOBS, repackData)
        except: repackJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_STREAM_FILES)
            repackJobIds = [int(line[0]) for line in repackData.data]
            streamerData = StreamerByRepackJobIdsDP(repackJobIds, params.orderby, params.asc, params.page)
            streamerFiles = getTableGui(T_STREAM_FILES, streamerData)
        except: streamerFiles = formatExceptionInfo()


        return self.templatePage (
                "history", {
                    T_STREAM_FILES: streamerFiles,
                    T_REPACK_JOBS : repackJobs,
                    T_REPACK_FILES : repackFiles,
                    T_MERGE_JOBS : mergeJobs,
                    T_MERGE_FILES : mergeFiles,
                    T_RECO_JOBS : recoJobs,
                    T_RECO_FILES : recoFiles,
                    T_RECO_MERGE_JOBS : recoMergeJobs,
                    T_RECO_MERGE_FILES : recoMergeFiles,
                    T_ALCA_JOBS : alcaJobs,
                    T_ALCA_FILES : alcaFiles,
                    T_ALCA_MERGE_JOBS : alcaMergeJobs,
                    T_ALCA_MERGE_FILES : alcaMergeFiles,
                    'entity' : entity + ' ' + str(id),
                    "tmplPath" : TMPL_PATH
                }
            )




    @expose
    def reco(self, id, **kwds):
        """
        Display reco file history
        """
        globals.pageParams = kwds
        entity = 'Temporary Reco file'

        try:
            id = int(id)
        except ValueError:
            return self.templatePage (
                "error", {
                    'msg': "Bad " + entity + " id",
                    "tmplPath" : TMPL_PATH
                }
            )

        try:
            params = self._getTableParams(T_RECO_FILES)
            recoData = RecoByIdsDP([id], params.orderby, params.asc, params.page)
            recoFiles = getTableGui(T_RECO_FILES, recoData)
            recoFiles = wrapInBorder(recoFiles)
        except:
            recoFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_MERGE_FILES)
            recoIds = [int(line[0]) for line in recoData.data]
            mergeFilesData = RepackMergeByRecoIdsDP(recoIds, params.orderby, params.asc, params.page)
            mergeFiles = getTableGui(T_MERGE_FILES, mergeFilesData)
        except: mergeFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_JOBS)
            recoJobData = RecoJobByRecoIdsDP(recoIds, params.orderby, params.asc, params.page)
            recoJobs = getTableGui(T_RECO_JOBS, recoJobData)
        except:
            recoJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_REPACK_FILES)
            mergeIds = [int(line[0]) for line in mergeFilesData.data]
            repackData = RepackByRepackMergeIdsDP(mergeIds, params.orderby, params.asc, params.page)
            repackFiles = getTableGui(T_REPACK_FILES, repackData)
        except: repackFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_MERGE_JOBS)
            mergeData = RepackMergeJobByRepackMergeIdsDP(mergeIds, params.orderby, params.asc, params.page)
            mergeJobs = getTableGui(T_MERGE_JOBS, mergeData)
        except: mergeJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_REPACK_JOBS)
            repackIds = [int(line[0]) for line in repackData.data]
            repackJobData = RepackJobByRepackIdsDP(repackIds, params.orderby, params.asc, params.page)
            repackJobs = getTableGui(T_REPACK_JOBS, repackJobData)
        except: repackJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_STREAM_FILES)
            repackJobIds = [int(line[0]) for line in repackJobData.data]
            streamerData = StreamerByRepackJobIdsDP(repackJobIds, params.orderby, params.asc, params.page)
            streamerFiles = getTableGui(T_STREAM_FILES, streamerData)
        except: streamerFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_MERGE_JOBS)
            recoMergeData = RecoMergeJobByRecoIdsDP(recoIds, params.orderby, params.asc, params.page)
            recoMergeJobs = getTableGui(T_RECO_MERGE_JOBS, recoMergeData)
        except:
            recoMergeJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_MERGE_FILES)
            recoMergeData = RecoMergeByRecoIdsDP(recoIds, params.orderby, params.asc, params.page)
            recoMergeFiles = getTableGui(T_RECO_MERGE_FILES, recoMergeData)
        except:
            recoMergeFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_ALCA_JOBS)
            recoMergeIds = [int(line[0]) for line in recoMergeData.data]
            alcaSkimData = AlcaSkimJobByRecoMergeIdsDP(recoMergeIds, params.orderby, params.asc, params.page)
            alcaJobs = getTableGui(T_ALCA_JOBS, alcaSkimData)
        except:
            alcaJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_ALCA_FILES)
            alcaSkimData = AlcaSkimByRecoMergeIdsDp(recoMergeIds, params.orderby, params.asc, params.page)
            alcaFiles = getTableGui(T_ALCA_FILES, alcaSkimData)
        except:
            alcaFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_ALCA_MERGE_JOBS)
            alcaSkimIds = [int(line[0]) for line in alcaSkimData.data]
            alcaSkimMergeData = AlcaSkimMergeJobByAlcaSkimIdsDP(alcaSkimIds, params.orderby, params.asc, params.page)
            alcaMergeJobs = getTableGui(T_ALCA_MERGE_FILES, alcaSkimMergeData)
        except:
            alcaMergeJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_ALCA_MERGE_FILES)
            alcaSkimMergeData = AlcaSkimMergeByAlcaSkimIdsDP(alcaSkimIds, params.orderby, params.asc, params.page)
            alcaMergeFiles = getTableGui(T_ALCA_MERGE_FILES, alcaSkimMergeData)
        except:
            alcaMergeFiles = formatExceptionInfo()

        return self.templatePage (
                "history", {
                    T_STREAM_FILES: streamerFiles,
                    T_REPACK_JOBS : repackJobs,
                    T_REPACK_FILES : repackFiles,
                    T_MERGE_JOBS : mergeJobs,
                    T_MERGE_FILES : mergeFiles,
                    T_RECO_JOBS : recoJobs,
                    T_RECO_FILES : recoFiles,
                    T_RECO_MERGE_JOBS : recoMergeJobs,
                    T_RECO_MERGE_FILES : recoMergeFiles,
                    T_ALCA_JOBS : alcaJobs,
                    T_ALCA_FILES : alcaFiles,
                    T_ALCA_MERGE_JOBS : alcaMergeJobs,
                    T_ALCA_MERGE_FILES : alcaMergeFiles,
                    'entity' : entity + ' ' + str(id),
                    "tmplPath" : TMPL_PATH
                }
            )

    @expose
    def recoMerge(self, id, **kwds):
        """
        Display reco merged file history
        """
        globals.pageParams = kwds
        entity = 'Reco file'

        try:
            id = int(id)
        except ValueError:
            return self.templatePage (
                "error", {
                    'msg': "Bad " + entity + " id",
                    "tmplPath" : TMPL_PATH
                }
            )

        try:
            params = self._getTableParams(T_RECO_MERGE_FILES)
            recoMergeData = RecoMergeByIdsDP([id], params.orderby, params.asc, params.page)
            recoMergeFiles = getTableGui(T_RECO_MERGE_FILES, recoMergeData)
	    recoMergeFiles = wrapInBorder(recoMergeFiles)
        except:
            recoMergeFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_FILES)
            recoMergeIds = [int(line[0]) for line in recoMergeData.data]
            recoData = RecoByRecoMergeIdsDP(recoMergeIds, params.orderby, params.asc, params.page)
            recoFiles = getTableGui(T_RECO_FILES, recoData)
        except:
            recoFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_MERGE_JOBS)
            recoMergeData = RecoMergeJobByRecoMergeIdsDP(recoMergeIds, params.orderby, params.asc, params.page)
            recoMergeJobs = getTableGui(T_RECO_MERGE_JOBS, recoMergeData)
        except:
            recoMergeJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_MERGE_FILES)
	    recoIds = [int(line[0]) for line in recoData.data]
            mergeFilesData = RepackMergeByRecoIdsDP(recoIds, params.orderby, params.asc, params.page)
            mergeFiles = getTableGui(T_MERGE_FILES, mergeFilesData)
        except: mergeFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_JOBS)
            recoJobData = RecoJobByRecoIdsDP(recoIds, params.orderby, params.asc, params.page)
            recoJobs = getTableGui(T_RECO_JOBS, recoJobData)
        except:
            recoJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_REPACK_FILES)
            mergeIds = [int(line[0]) for line in mergeFilesData.data]
            repackData = RepackByRepackMergeIdsDP(mergeIds, params.orderby, params.asc, params.page)
            repackFiles = getTableGui(T_REPACK_FILES, repackData)
        except: repackFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_MERGE_JOBS)
            mergeData = RepackMergeJobByRepackMergeIdsDP(mergeIds, params.orderby, params.asc, params.page)
            mergeJobs = getTableGui(T_MERGE_JOBS, mergeData)
        except: mergeJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_REPACK_JOBS)
            repackIds = [int(line[0]) for line in repackData.data]
            repackJobData = RepackJobByRepackIdsDP(repackIds, params.orderby, params.asc, params.page)
            repackJobs = getTableGui(T_REPACK_JOBS, repackJobData)
        except: repackJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_STREAM_FILES)
            repackJobIds = [int(line[0]) for line in repackJobData.data]
            streamerData = StreamerByRepackJobIdsDP(repackJobIds, params.orderby, params.asc, params.page)
            streamerFiles = getTableGui(T_STREAM_FILES, streamerData)
        except: streamerFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_ALCA_JOBS)
            alcaSkimData = AlcaSkimJobByRecoMergeIdsDP(recoMergeIds, params.orderby, params.asc, params.page)
            alcaJobs = getTableGui(T_ALCA_JOBS, alcaSkimData)
        except:
            alcaJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_ALCA_FILES)
            alcaSkimData = AlcaSkimByRecoMergeIdsDp(recoMergeIds, params.orderby, params.asc, params.page)
            alcaFiles = getTableGui(T_ALCA_FILES, alcaSkimData)
        except:
            alcaFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_ALCA_MERGE_JOBS)
            alcaSkimIds = [int(line[0]) for line in alcaSkimData.data]
            alcaSkimMergeData = AlcaSkimMergeJobByAlcaSkimIdsDP(alcaSkimIds, params.orderby, params.asc, params.page)
            alcaMergeJobs = getTableGui(T_ALCA_MERGE_FILES, alcaSkimMergeData)
        except:
            alcaMergeJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_ALCA_MERGE_FILES)
            alcaSkimMergeData = AlcaSkimMergeByAlcaSkimIdsDP(alcaSkimIds, params.orderby, params.asc, params.page)
            alcaMergeFiles = getTableGui(T_ALCA_MERGE_FILES, alcaSkimMergeData)
        except:
            alcaMergeFiles = formatExceptionInfo()

        return self.templatePage (
                "history", {
                    T_STREAM_FILES: streamerFiles,
                    T_REPACK_JOBS : repackJobs,
                    T_REPACK_FILES : repackFiles,
                    T_MERGE_JOBS : mergeJobs,
                    T_MERGE_FILES : mergeFiles,
                    T_RECO_JOBS : recoJobs,
                    T_RECO_FILES : recoFiles,
                    T_RECO_MERGE_JOBS : recoMergeJobs,
                    T_RECO_MERGE_FILES : recoMergeFiles,
                    T_ALCA_JOBS : alcaJobs,
                    T_ALCA_FILES : alcaFiles,
                    T_ALCA_MERGE_JOBS : alcaMergeJobs,
                    T_ALCA_MERGE_FILES : alcaMergeFiles,
                    'entity' : entity + ' ' + str(id),
                    "tmplPath" : TMPL_PATH
                }
            )


    @expose
    def alcaSkim(self, id, **kwds):
        """
        Display alcaSkim file history
        """
        globals.pageParams = kwds
        entity = 'Temporary AlcaSkim file'

        try:
            id = int(id)
        except ValueError:
            return self.templatePage (
                "error", {
                    'msg': "Bad " + entity + " id",
                    "tmplPath" : TMPL_PATH
                }
            )

        try:
            params = self._getTableParams(T_ALCA_FILES)
            alcaSkimData = AlcaSkimByIdsDP([id], params.orderby, params.asc, params.page)
            alcaFiles = getTableGui(T_ALCA_FILES, alcaSkimData)
	    alcaFiles = wrapInBorder(alcaFiles)
        except:
            alcaFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_ALCA_JOBS)
            alcaSkimIds = [int(line[0]) for line in alcaSkimData.data]
            alcaSkimData = AlcaSkimJobByAlcaSkimIdsDP(alcaSkimIds, params.orderby, params.asc, params.page)
            alcaJobs = getTableGui(T_ALCA_JOBS, alcaSkimData)
        except:
            alcaJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_MERGE_FILES)
            recoMergeData = RecoMergeByAlcaSkimIdsDP(alcaSkimIds, params.orderby, params.asc, params.page)
            recoMergeFiles = getTableGui(T_RECO_MERGE_FILES, recoMergeData)
        except:
            recoMergeFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_FILES)
            recoMergeIds = [int(line[0]) for line in recoMergeData.data]
            recoData = RecoByRecoMergeIdsDP(recoMergeIds, params.orderby, params.asc, params.page)
            recoFiles = getTableGui(T_RECO_FILES, recoData)
        except:
            recoFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_MERGE_JOBS)
            recoMergeData = RecoMergeJobByRecoMergeIdsDP(recoMergeIds, params.orderby, params.asc, params.page)
            recoMergeJobs = getTableGui(T_RECO_MERGE_JOBS, recoMergeData)
        except:
            recoMergeJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_MERGE_FILES)
	    recoIds = [int(line[0]) for line in recoData.data]
            mergeFilesData = RepackMergeByRecoIdsDP(recoIds, params.orderby, params.asc, params.page)
            mergeFiles = getTableGui(T_MERGE_FILES, mergeFilesData)
        except: mergeFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_JOBS)
            recoJobData = RecoJobByRecoIdsDP(recoIds, params.orderby, params.asc, params.page)    
            recoJobs = getTableGui(T_RECO_JOBS, recoJobData)
        except:
            recoJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_REPACK_FILES)
            mergeIds = [int(line[0]) for line in mergeFilesData.data]
            repackData = RepackByRepackMergeIdsDP(mergeIds, params.orderby, params.asc, params.page)
            repackFiles = getTableGui(T_REPACK_FILES, repackData)
        except: repackFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_MERGE_JOBS)
            mergeData = RepackMergeJobByRepackMergeIdsDP(mergeIds, params.orderby, params.asc, params.page)
            mergeJobs = getTableGui(T_MERGE_JOBS, mergeData)
        except: mergeJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_REPACK_JOBS)
            repackIds = [int(line[0]) for line in repackData.data]
            repackJobData = RepackJobByRepackIdsDP(repackIds, params.orderby, params.asc, params.page)
            repackJobs = getTableGui(T_REPACK_JOBS, repackJobData)
        except: repackJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_STREAM_FILES)
            repackJobIds = [int(line[0]) for line in repackJobData.data]
            streamerData = StreamerByRepackJobIdsDP(repackJobIds, params.orderby, params.asc, params.page)
            streamerFiles = getTableGui(T_STREAM_FILES, streamerData)
        except: streamerFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_ALCA_MERGE_JOBS)
            alcaSkimMergeData = AlcaSkimMergeJobByAlcaSkimIdsDP(alcaSkimIds, params.orderby, params.asc, params.page)
            alcaMergeJobs = getTableGui(T_ALCA_MERGE_FILES, alcaSkimMergeData)
        except:
            alcaMergeJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_ALCA_MERGE_FILES)
            alcaSkimMergeData = AlcaSkimMergeByAlcaSkimIdsDP(alcaSkimIds, params.orderby, params.asc, params.page)
            alcaMergeFiles = getTableGui(T_ALCA_MERGE_FILES, alcaSkimMergeData)
        except:
            alcaMergeFiles = formatExceptionInfo()

        return self.templatePage (
                "history", {
                    T_STREAM_FILES: streamerFiles,
                    T_REPACK_JOBS : repackJobs,
                    T_REPACK_FILES : repackFiles,
                    T_MERGE_JOBS : mergeJobs,
                    T_MERGE_FILES : mergeFiles,
                    T_RECO_JOBS : recoJobs,
                    T_RECO_FILES : recoFiles,
                    T_RECO_MERGE_JOBS : recoMergeJobs,
                    T_RECO_MERGE_FILES : recoMergeFiles,
                    T_ALCA_JOBS : alcaJobs,
                    T_ALCA_FILES : alcaFiles,
                    T_ALCA_MERGE_JOBS : alcaMergeJobs,
                    T_ALCA_MERGE_FILES : alcaMergeFiles,
                    'entity' : entity + ' ' + str(id),
                    "tmplPath" : TMPL_PATH
                }
            )


    @expose
    def alcaSkimMerge(self, id, **kwds):
        """
        Display alcaSkim merged file history
        """
        globals.pageParams = kwds
        entity = 'AlcaSkim file'

        try:
            id = int(id)
        except ValueError:
            return self.templatePage (
                "error", {
                    'msg': "Bad " + entity + " id",
                    "tmplPath" : TMPL_PATH
                }
            )

        try:
            params = self._getTableParams(T_ALCA_MERGE_FILES)
            alcaSkimMergeData = AlcaSkimMergeByIdsDP([id], params.orderby, params.asc, params.page)
            alcaMergeFiles = getTableGui(T_ALCA_MERGE_FILES, alcaSkimMergeData)
	    alcaMergeFiles = wrapInBorder(alcaMergeFiles)
        except:
            alcaMergeFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_ALCA_MERGE_JOBS)
	    alcaSkimMergeIds = [int(line[0]) for line in alcaSkimMergeData.data]
            alcaSkimMergeData = AlcaSkimMergeJobByAlcaSkimMergeIdsDP(alcaSkimMergeIds, params.orderby, params.asc, params.page)
            alcaMergeJobs = getTableGui(T_ALCA_MERGE_FILES, alcaSkimMergeData)
        except:
            alcaMergeJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_ALCA_FILES)
            alcaSkimData = AlcaSkimByAlcaSkimMergeIdsDP(alcaSkimMergeIds, params.orderby, params.asc, params.page)
            alcaFiles = getTableGui(T_ALCA_FILES, alcaSkimData)
        except:
            alcaFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_ALCA_JOBS)
	    alcaSkimIds = [int(line[0]) for line in alcaSkimData.data]
            alcaSkimData = AlcaSkimJobByAlcaSkimIdsDP(alcaSkimIds, params.orderby, params.asc, params.page)
            alcaJobs = getTableGui(T_ALCA_JOBS, alcaSkimData)
        except:
            alcaJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_MERGE_FILES)
            recoMergeData = RecoMergeByAlcaSkimIdsDP(alcaSkimIds, params.orderby, params.asc, params.page)
            recoMergeFiles = getTableGui(T_RECO_MERGE_FILES, recoMergeData)
        except:
            recoMergeFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_FILES)
            recoMergeIds = [int(line[0]) for line in recoMergeData.data]
            recoData = RecoByRecoMergeIdsDP(recoMergeIds, params.orderby, params.asc, params.page)
            recoFiles = getTableGui(T_RECO_FILES, recoData)
        except:
            recoFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_MERGE_JOBS)
            recoMergeData = RecoMergeJobByRecoMergeIdsDP(recoMergeIds, params.orderby, params.asc, params.page)
            recoMergeJobs = getTableGui(T_RECO_MERGE_JOBS, recoMergeData)
        except:
            recoMergeJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_MERGE_FILES)
            recoIds = [int(line[0]) for line in recoData.data]
            mergeFilesData = RepackMergeByRecoIdsDP(recoIds, params.orderby, params.asc, params.page)
            mergeFiles = getTableGui(T_MERGE_FILES, mergeFilesData)
        except: mergeFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_JOBS)
            recoJobData = RecoJobByRecoIdsDP(recoIds, params.orderby, params.asc, params.page)
            recoJobs = getTableGui(T_RECO_JOBS, recoJobData)
        except:
            recoJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_REPACK_FILES)
            mergeIds = [int(line[0]) for line in mergeFilesData.data]
            repackData = RepackByRepackMergeIdsDP(mergeIds, params.orderby, params.asc, params.page)
            repackFiles = getTableGui(T_REPACK_FILES, repackData)
        except: repackFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_MERGE_JOBS)
            mergeData = RepackMergeJobByRepackMergeIdsDP(mergeIds, params.orderby, params.asc, params.page)
            mergeJobs = getTableGui(T_MERGE_JOBS, mergeData)
        except: mergeJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_REPACK_JOBS)
            repackIds = [int(line[0]) for line in repackData.data]
            repackJobData = RepackJobByRepackIdsDP(repackIds, params.orderby, params.asc, params.page)
            repackJobs = getTableGui(T_REPACK_JOBS, repackJobData)
        except: repackJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_STREAM_FILES)
            repackJobIds = [int(line[0]) for line in repackJobData.data]
            streamerData = StreamerByRepackJobIdsDP(repackJobIds, params.orderby, params.asc, params.page)
            streamerFiles = getTableGui(T_STREAM_FILES, streamerData)
        except: streamerFiles = formatExceptionInfo()

        return self.templatePage (
                "history", {
                    T_STREAM_FILES: streamerFiles,
                    T_REPACK_JOBS : repackJobs,
                    T_REPACK_FILES : repackFiles,
                    T_MERGE_JOBS : mergeJobs,
                    T_MERGE_FILES : mergeFiles,
                    T_RECO_JOBS : recoJobs,
                    T_RECO_FILES : recoFiles,
                    T_RECO_MERGE_JOBS : recoMergeJobs,
                    T_RECO_MERGE_FILES : recoMergeFiles,
                    T_ALCA_JOBS : alcaJobs,
                    T_ALCA_FILES : alcaFiles,
                    T_ALCA_MERGE_JOBS : alcaMergeJobs,
                    T_ALCA_MERGE_FILES : alcaMergeFiles,
                    'entity' : entity + ' ' + str(id),
                    "tmplPath" : TMPL_PATH
                }
            )



    @expose
    def recoJob(self, id, **kwds):
        """
        Display reco job history
        """
        globals.pageParams = kwds
        entity = 'Reco job'

        try:
            id = int(id)
        except ValueError:
            return self.templatePage (
                "error", {
                    'msg': "Bad " + entity + " id",
                    "tmplPath" : TMPL_PATH
                }
            )

        try:
            params = self._getTableParams(T_RECO_JOBS)
            recoJobData = RecoJobByIdsDP([id], params.orderby, params.asc, params.page)
            recoJobs = getTableGui(T_RECO_JOBS, recoJobData)
            recoJobs = wrapInBorder(recoJobs)
        except:
            recoJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_MERGE_FILES)
            recoJobIds = [int(line[0]) for line in recoJobData.data]
            mergeFilesData = RepackMergeByRecoJobIdsDP(recoJobIds, params.orderby, params.asc, params.page)
            mergeFiles = getTableGui(T_MERGE_FILES, mergeFilesData)
        except: mergeFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_MERGE_JOBS)
            mergeIds = [int(line[0]) for line in mergeFilesData.data]
            mergeData = RepackMergeJobByRepackMergeIdsDP(mergeIds, params.orderby, params.asc, params.page)
            mergeJobs = getTableGui(T_MERGE_JOBS, mergeData)
        except: mergeJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_REPACK_FILES)
            repackData = RepackByRepackMergeIdsDP(mergeIds, params.orderby, params.asc, params.page)
            repackFiles = getTableGui(T_REPACK_FILES, repackData)
        except: repackFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_REPACK_JOBS)
            repackIds = [int(line[0]) for line in repackData.data]
            repackData = RepackJobByRepackIdsDP(repackIds, params.orderby, params.asc, params.page)
            repackJobs = getTableGui(T_REPACK_JOBS, repackData)
        except: repackJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_STREAM_FILES)
            repackJobIds = [int(line[0]) for line in repackData.data]
            streamerData = StreamerByRepackJobIdsDP(repackJobIds, params.orderby, params.asc, params.page)
            streamerFiles = getTableGui(T_STREAM_FILES, streamerData)
        except: streamerFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_FILES)
            recoData = RecoByMergeIdsDP(mergeIds, params.orderby, params.asc, params.page)
            recoFiles = getTableGui(T_RECO_FILES, recoData)
        except:
            recoFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_MERGE_JOBS)
            recoIds = [int(line[0]) for line in recoData.data]
            recoMergeData = RecoMergeJobByRecoIdsDP(recoIds, params.orderby, params.asc, params.page)
            recoMergeJobs = getTableGui(T_RECO_MERGE_JOBS, recoMergeData)
        except:
            recoMergeJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_MERGE_FILES)
            recoMergeData = RecoMergeByRecoIdsDP(recoIds, params.orderby, params.asc, params.page)
            recoMergeFiles = getTableGui(T_RECO_MERGE_FILES, recoMergeData)
        except:
            recoMergeFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_ALCA_JOBS)
            recoMergeIds = [int(line[0]) for line in recoMergeData.data]
            alcaSkimData = AlcaSkimJobByRecoMergeIdsDP(recoMergeIds, params.orderby, params.asc, params.page)
            alcaJobs = getTableGui(T_ALCA_JOBS, alcaSkimData)
        except:
            alcaJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_ALCA_FILES)
            alcaSkimData = AlcaSkimByRecoMergeIdsDp(recoMergeIds, params.orderby, params.asc, params.page)
            alcaFiles = getTableGui(T_ALCA_FILES, alcaSkimData)
        except:
            alcaFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_ALCA_MERGE_JOBS)
            alcaSkimIds = [int(line[0]) for line in alcaSkimData.data]
            alcaSkimMergeData = AlcaSkimMergeJobByAlcaSkimIdsDP(alcaSkimIds, params.orderby, params.asc, params.page)
            alcaMergeJobs = getTableGui(T_ALCA_MERGE_FILES, alcaSkimMergeData)
        except:
            alcaMergeJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_ALCA_MERGE_FILES)
            alcaSkimMergeData = AlcaSkimMergeByAlcaSkimIdsDP(alcaSkimIds, params.orderby, params.asc, params.page)
            alcaMergeFiles = getTableGui(T_ALCA_MERGE_FILES, alcaSkimMergeData)
        except:
            alcaMergeFiles = formatExceptionInfo()

        return self.templatePage (
                "history", {
                    T_STREAM_FILES: streamerFiles,
                    T_REPACK_JOBS : repackJobs,
                    T_REPACK_FILES : repackFiles,
                    T_MERGE_JOBS : mergeJobs,
                    T_MERGE_FILES : mergeFiles,
                    T_RECO_JOBS : recoJobs,
                    T_RECO_FILES : recoFiles,
                    T_RECO_MERGE_JOBS : recoMergeJobs,
                    T_RECO_MERGE_FILES : recoMergeFiles,
                    T_ALCA_JOBS : alcaJobs,
                    T_ALCA_FILES : alcaFiles,
                    T_ALCA_MERGE_JOBS : alcaMergeJobs,
                    T_ALCA_MERGE_FILES : alcaMergeFiles,
                    'entity' : entity + ' ' + str(id),
                    "tmplPath" : TMPL_PATH
                }
            )

    @expose
    def repackMerge(self, id, **kwds):
        """
        Display repack merge file history
        """
        globals.pageParams = kwds
        entity = 'Repacked file'

        try:
            id = int(id)
        except ValueError:
            return self.templatePage (
                "error", {
                    'msg': "Bad " + entity + " id",
                    "tmplPath" : TMPL_PATH
                }
            )

        try:
            params = self._getTableParams(T_MERGE_FILES)
            mergeFilesData = RepackMergeByIdsDP([id], params.orderby, params.asc, params.page)
            mergeFiles = getTableGui(T_MERGE_FILES, mergeFilesData)
            mergeFiles = wrapInBorder(mergeFiles)
        except: mergeFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_REPACK_FILES)
            mergeIds = [int(line[0]) for line in mergeFilesData.data]
            repackData = RepackByRepackMergeIdsDP(mergeIds, params.orderby, params.asc, params.page)
            repackFiles = getTableGui(T_REPACK_FILES, repackData)
        except: repackFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_MERGE_JOBS)
            mergeData = RepackMergeJobByRepackMergeIdsDP(mergeIds, params.orderby, params.asc, params.page)
            mergeJobs = getTableGui(T_MERGE_JOBS, mergeData)
        except: mergeJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_REPACK_JOBS)
            repackIds = [int(line[0]) for line in repackData.data]
            repackJobData = RepackJobByRepackIdsDP(repackIds, params.orderby, params.asc, params.page)
            repackJobs = getTableGui(T_REPACK_JOBS, repackJobData)
        except: repackJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_STREAM_FILES)
            repackJobIds = [int(line[0]) for line in repackJobData.data]
            streamerData = StreamerByRepackJobIdsDP(repackJobIds, params.orderby, params.asc, params.page)
            streamerFiles = getTableGui(T_STREAM_FILES, streamerData)
        except: streamerFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_JOBS)
            recoJobData = RecoJobByMergeIdsDP(mergeIds, params.orderby, params.asc, params.page)
            recoJobs = getTableGui(T_RECO_JOBS, recoJobData)
        except:
            recoJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_FILES)
            recoData = RecoByMergeIdsDP(mergeIds, params.orderby, params.asc, params.page)
            recoFiles = getTableGui(T_RECO_FILES, recoData)
        except:
            recoFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_MERGE_JOBS)
            recoIds = [int(line[0]) for line in recoData.data]
            recoMergeData = RecoMergeJobByRecoIdsDP(recoIds, params.orderby, params.asc, params.page)
            recoMergeJobs = getTableGui(T_RECO_MERGE_JOBS, recoMergeData)
        except:
            recoMergeJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_MERGE_FILES)
            recoMergeData = RecoMergeByRecoIdsDP(recoIds, params.orderby, params.asc, params.page)
            recoMergeFiles = getTableGui(T_RECO_MERGE_FILES, recoMergeData)
        except:
            recoMergeFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_ALCA_JOBS)
            recoMergeIds = [int(line[0]) for line in recoMergeData.data]
            alcaSkimData = AlcaSkimJobByRecoMergeIdsDP(recoMergeIds, params.orderby, params.asc, params.page)
            alcaJobs = getTableGui(T_ALCA_JOBS, alcaSkimData)
        except:
            alcaJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_ALCA_FILES)
            alcaSkimData = AlcaSkimByRecoMergeIdsDp(recoMergeIds, params.orderby, params.asc, params.page)
            alcaFiles = getTableGui(T_ALCA_FILES, alcaSkimData)
        except:
            alcaFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_ALCA_MERGE_JOBS)
            alcaSkimIds = [int(line[0]) for line in alcaSkimData.data]
            alcaSkimMergeData = AlcaSkimMergeJobByAlcaSkimIdsDP(alcaSkimIds, params.orderby, params.asc, params.page)
            alcaMergeJobs = getTableGui(T_ALCA_MERGE_FILES, alcaSkimMergeData)
        except:
            alcaMergeJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_ALCA_MERGE_FILES)
            alcaSkimMergeData = AlcaSkimMergeByAlcaSkimIdsDP(alcaSkimIds, params.orderby, params.asc, params.page)
            alcaMergeFiles = getTableGui(T_ALCA_MERGE_FILES, alcaSkimMergeData)
        except:
            alcaMergeFiles = formatExceptionInfo()

        return self.templatePage (
                "history", {
                    T_STREAM_FILES: streamerFiles,
                    T_REPACK_JOBS : repackJobs,
                    T_REPACK_FILES : repackFiles,
                    T_MERGE_JOBS : mergeJobs,
                    T_MERGE_FILES : mergeFiles,
                    T_RECO_JOBS : recoJobs,
                    T_RECO_FILES : recoFiles,
                    T_RECO_MERGE_JOBS : recoMergeJobs,
                    T_RECO_MERGE_FILES : recoMergeFiles,
                    T_ALCA_JOBS : alcaJobs,
                    T_ALCA_FILES : alcaFiles,
                    T_ALCA_MERGE_JOBS : alcaMergeJobs,
                    T_ALCA_MERGE_FILES : alcaMergeFiles,
                    'entity' : entity + ' ' + str(id),
                    "tmplPath" : TMPL_PATH
                }
            )

    @expose
    def repackMergeJob(self, id, **kwds):
        """
        Display repack merge job history
        """
        globals.pageParams = kwds
        entity = 'Repack Merge job'

        try:
            id = int(id)
        except ValueError:
            return self.templatePage (
                "error", {
                    'msg': "Bad " + entity + " id",
                    "tmplPath" : TMPL_PATH
                }
            )

        try:
            params = self._getTableParams(T_MERGE_JOBS)
            mergeData = RepackMergeJobByIdsDP([id], params.orderby, params.asc, params.page)
            mergeJobs = getTableGui(T_MERGE_JOBS, mergeData)
            mergeJobs = wrapInBorder(mergeJobs)
        except: mergeJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_REPACK_FILES)
            mergeJobIds = [int(line[0]) for line in mergeData.data]
            repackData = RepackByRepackMergeJobIdsDP(mergeJobIds, params.orderby, params.asc, params.page)
            repackFiles = getTableGui(T_REPACK_FILES, repackData)
        except: repackFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_REPACK_JOBS)
            repackIds = [int(line[0]) for line in repackData.data]
            repackData = RepackJobByRepackIdsDP(repackIds, params.orderby, params.asc, params.page)
            repackJobs = getTableGui(T_REPACK_JOBS, repackData)
        except: repackJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_STREAM_FILES)
            repackJobIds = [int(line[0]) for line in repackData.data]
            streamerData = StreamerByRepackJobIdsDP(repackJobIds, params.orderby, params.asc, params.page)
            streamerFiles = getTableGui(T_STREAM_FILES, streamerData)
        except: streamerFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_MERGE_FILES)
            mergeFilesData = RepackMergeByRepackIdsDP(repackIds, params.orderby, params.asc, params.page)
            mergeFiles = getTableGui(T_MERGE_FILES, mergeFilesData)
        except: mergeFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_JOBS)
            mergeIds = [int(line[0]) for line in mergeFilesData.data]
            recoJobData = RecoJobByMergeIdsDP(mergeIds, params.orderby, params.asc, params.page)
            recoJobs = getTableGui(T_RECO_JOBS, recoJobData)
        except:
            recoJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_FILES)
            recoData = RecoByMergeIdsDP(mergeIds, params.orderby, params.asc, params.page)
            recoFiles = getTableGui(T_RECO_FILES, recoData)
        except:
            recoFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_MERGE_JOBS)
            recoIds = [int(line[0]) for line in recoData.data]
            recoMergeData = RecoMergeJobByRecoIdsDP(recoIds, params.orderby, params.asc, params.page)
            recoMergeJobs = getTableGui(T_RECO_MERGE_JOBS, recoMergeData)
        except:
            recoMergeJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_MERGE_FILES)
            recoMergeData = RecoMergeByRecoIdsDP(recoIds, params.orderby, params.asc, params.page)
            recoMergeFiles = getTableGui(T_RECO_MERGE_FILES, recoMergeData)
        except:
            recoMergeFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_ALCA_JOBS)
            recoMergeIds = [int(line[0]) for line in recoMergeData.data]
            alcaSkimData = AlcaSkimJobByRecoMergeIdsDP(recoMergeIds, params.orderby, params.asc, params.page)
            alcaJobs = getTableGui(T_ALCA_JOBS, alcaSkimData)
        except:
            alcaJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_ALCA_FILES)
            alcaSkimData = AlcaSkimByRecoMergeIdsDp(recoMergeIds, params.orderby, params.asc, params.page)
            alcaFiles = getTableGui(T_ALCA_FILES, alcaSkimData)
        except:
            alcaFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_ALCA_MERGE_JOBS)
            alcaSkimIds = [int(line[0]) for line in alcaSkimData.data]
            alcaSkimMergeData = AlcaSkimMergeJobByAlcaSkimIdsDP(alcaSkimIds, params.orderby, params.asc, params.page)
            alcaMergeJobs = getTableGui(T_ALCA_MERGE_FILES, alcaSkimMergeData)
        except:
            alcaMergeJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_ALCA_MERGE_FILES)
            alcaSkimMergeData = AlcaSkimMergeByAlcaSkimIdsDP(alcaSkimIds, params.orderby, params.asc, params.page)
            alcaMergeFiles = getTableGui(T_ALCA_MERGE_FILES, alcaSkimMergeData)
        except:
            alcaMergeFiles = formatExceptionInfo()

        return self.templatePage (
                "history", {
                    T_STREAM_FILES: streamerFiles,
                    T_REPACK_JOBS : repackJobs,
                    T_REPACK_FILES : repackFiles,
                    T_MERGE_JOBS : mergeJobs,
                    T_MERGE_FILES : mergeFiles,
                    T_RECO_JOBS : recoJobs,
                    T_RECO_FILES : recoFiles,
                    T_RECO_MERGE_JOBS : recoMergeJobs,
                    T_RECO_MERGE_FILES : recoMergeFiles,
                    T_ALCA_JOBS : alcaJobs,
                    T_ALCA_FILES : alcaFiles,
                    T_ALCA_MERGE_JOBS : alcaMergeJobs,
                    T_ALCA_MERGE_FILES : alcaMergeFiles,
                    'entity' : entity + ' ' + str(id),
                    "tmplPath" : TMPL_PATH
                }
            )

    @expose
    def streamer(self, id, **kwds):
        """
        Display streamer file history
        """
        globals.pageParams = kwds
        entity = "Streamer file"

        try:
            id = int(id)
        except ValueError:
            return self.templatePage (
                "error", {
                    'msg': "Bad " + entity + " id",
                    "tmplPath" : TMPL_PATH
                }
            )

        try:
            params = self._getTableParams(T_STREAM_FILES)
            streamerData = StreamerByIdsDP([id], params.orderby, params.asc, params.page)
            streamerFiles = getTableGui(T_STREAM_FILES, streamerData)
            streamerFiles = wrapInBorder(streamerFiles)
        except: streamerFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_REPACK_JOBS)
            streamerIds = [int(line[0]) for line in streamerData.data]
            repackData = RepackJobByStreamerDP(streamerIds, params.orderby, params.asc, params.page)
            repackJobs = getTableGui(T_REPACK_JOBS, repackData)
        except: repackJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_REPACK_FILES)
            repackData = RepackByStreamerIdsDP(streamerIds, params.orderby, params.asc, params.page)
            repackFiles = getTableGui(T_REPACK_FILES, repackData)
        except: repackFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_MERGE_JOBS)
            repackIds = [int(line[0]) for line in repackData.data]
            mergeData = RepackMergeJobByRepackIdsDP(repackIds, params.orderby, params.asc, params.page)
            mergeJobs = getTableGui(T_MERGE_JOBS, mergeData)
        except: mergeJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_MERGE_FILES)
            mergeData = RepackMergeByRepackIdsDP(repackIds, params.orderby, params.asc, params.page)
            mergeFiles = getTableGui(T_MERGE_FILES, mergeData)
        except: mergeFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_JOBS)
            mergeIds = [int(line[0]) for line in mergeData.data]
            recoData = RecoJobByMergeIdsDP(mergeIds, params.orderby, params.asc, params.page)
            recoJobs = getTableGui(T_RECO_JOBS, recoData)
        except:
            recoJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_FILES)
            recoData = RecoByMergeIdsDP(mergeIds, params.orderby, params.asc, params.page)
            recoFiles = getTableGui(T_RECO_FILES, recoData)
        except:
            recoFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_MERGE_JOBS)
            recoIds = [int(line[0]) for line in recoData.data]
            recoMergeData = RecoMergeJobByRecoIdsDP(recoIds, params.orderby, params.asc, params.page)
            recoMergeJobs = getTableGui(T_RECO_MERGE_JOBS, recoMergeData)
        except:
            recoMergeJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_MERGE_FILES)
            recoMergeData = RecoMergeByRecoIdsDP(recoIds, params.orderby, params.asc, params.page)
            recoMergeFiles = getTableGui(T_RECO_MERGE_FILES, recoMergeData)
        except:
            recoMergeFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_ALCA_JOBS)
            recoMergeIds = [int(line[0]) for line in recoMergeData.data]
            alcaSkimData = AlcaSkimJobByRecoMergeIdsDP(recoMergeIds, params.orderby, params.asc, params.page)
            alcaJobs = getTableGui(T_ALCA_JOBS, alcaSkimData)
        except:
            alcaJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_ALCA_FILES)
            alcaSkimData = AlcaSkimByRecoMergeIdsDp(recoMergeIds, params.orderby, params.asc, params.page)
            alcaFiles = getTableGui(T_ALCA_FILES, alcaSkimData)
        except:
            alcaFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_ALCA_MERGE_JOBS)
            alcaSkimIds = [int(line[0]) for line in alcaSkimData.data]
            alcaSkimMergeData = AlcaSkimMergeJobByAlcaSkimIdsDP(alcaSkimIds, params.orderby, params.asc, params.page)
            alcaMergeJobs = getTableGui(T_ALCA_MERGE_FILES, alcaSkimMergeData)
        except:
            alcaMergeJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_ALCA_MERGE_FILES)
            alcaSkimMergeData = AlcaSkimMergeByAlcaSkimIdsDP(alcaSkimIds, params.orderby, params.asc, params.page)
            alcaMergeFiles = getTableGui(T_ALCA_MERGE_FILES, alcaSkimMergeData)
        except:
            alcaMergeFiles = formatExceptionInfo()

        return self.templatePage (
                "history", {
                    T_STREAM_FILES: streamerFiles,
                    T_REPACK_JOBS : repackJobs,
                    T_REPACK_FILES : repackFiles,
                    T_MERGE_JOBS : mergeJobs,
                    T_MERGE_FILES : mergeFiles,
                    T_RECO_JOBS : recoJobs,
                    T_RECO_FILES : recoFiles,
		    T_RECO_MERGE_JOBS : recoMergeJobs,
		    T_RECO_MERGE_FILES : recoMergeFiles,
		    T_ALCA_JOBS : alcaJobs,
		    T_ALCA_FILES : alcaFiles,
		    T_ALCA_MERGE_JOBS : alcaMergeJobs,
		    T_ALCA_MERGE_FILES : alcaMergeFiles,
                    'entity' : entity + ' ' + str(id),
                    "tmplPath" : TMPL_PATH
                }
            )


    @expose
    def repack(self, id, **kwds):
        """
        Display repack file history
        """
        globals.pageParams = kwds
        entity = 'Temporary Repacked file'

        try:
            id = int(id)
        except ValueError:
            return self.templatePage (
                "error", {
                    'msg': "Bad " + entity + " id",
                    "tmplPath" : TMPL_PATH
                }
            )

        try:
            params = self._getTableParams(T_REPACK_FILES)
            repackData = RepackByIdsDP([id], params.orderby, params.asc, params.page)
            repackFiles = getTableGui(T_REPACK_FILES, repackData)
            repackFiles = wrapInBorder(repackFiles)
        except: repackFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_REPACK_JOBS)
            repackIds = [int(line[0]) for line in repackData.data]
            repackJobData = RepackJobByRepackIdsDP(repackIds, params.orderby, params.asc, params.page)
            repackJobs = getTableGui(T_REPACK_JOBS, repackJobData)
        except: repackJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_STREAM_FILES)
            repackJobIds = [int(line[0]) for line in repackJobData.data]
            streamerData = StreamerByRepackJobIdsDP(repackJobIds, params.orderby, params.asc, params.page)
            streamerFiles = getTableGui(T_STREAM_FILES, streamerData)
        except: streamerFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_MERGE_JOBS)
            mergeData = RepackMergeJobByRepackIdsDP(repackIds, params.orderby, params.asc, params.page)
            mergeJobs = getTableGui(T_MERGE_JOBS, mergeData)
        except: mergeJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_MERGE_FILES)
            mergeData = RepackMergeByRepackIdsDP(repackIds, params.orderby, params.asc, params.page)
            mergeFiles = getTableGui(T_MERGE_FILES, mergeData)
        except: mergeFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_JOBS)
            mergeIds = [int(line[0]) for line in mergeData.data]
            recoData = RecoJobByMergeIdsDP(mergeIds, params.orderby, params.asc, params.page)
            recoJobs = getTableGui(T_RECO_JOBS, recoData)
        except:
            recoJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_FILES)
            recoData = RecoByMergeIdsDP(mergeIds, params.orderby, params.asc, params.page)
            recoFiles = getTableGui(T_RECO_FILES, recoData)
        except:
            recoFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_MERGE_JOBS)
            recoIds = [int(line[0]) for line in recoData.data]
            recoMergeData = RecoMergeJobByRecoIdsDP(recoIds, params.orderby, params.asc, params.page)
            recoMergeJobs = getTableGui(T_RECO_MERGE_JOBS, recoMergeData)
        except:
            recoMergeJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_MERGE_FILES)
            recoMergeData = RecoMergeByRecoIdsDP(recoIds, params.orderby, params.asc, params.page)
            recoMergeFiles = getTableGui(T_RECO_MERGE_FILES, recoMergeData)
        except:
            recoMergeFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_ALCA_JOBS)
            recoMergeIds = [int(line[0]) for line in recoMergeData.data]
            alcaSkimData = AlcaSkimJobByRecoMergeIdsDP(recoMergeIds, params.orderby, params.asc, params.page)
            alcaJobs = getTableGui(T_ALCA_JOBS, alcaSkimData)
        except:
            alcaJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_ALCA_FILES)
            alcaSkimData = AlcaSkimByRecoMergeIdsDp(recoMergeIds, params.orderby, params.asc, params.page)
            alcaFiles = getTableGui(T_ALCA_FILES, alcaSkimData)
        except:
            alcaFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_ALCA_MERGE_JOBS)
            alcaSkimIds = [int(line[0]) for line in alcaSkimData.data]
            alcaSkimMergeData = AlcaSkimMergeJobByAlcaSkimIdsDP(alcaSkimIds, params.orderby, params.asc, params.page)
            alcaMergeJobs = getTableGui(T_ALCA_MERGE_FILES, alcaSkimMergeData)
        except:
            alcaMergeJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_ALCA_MERGE_FILES)
            alcaSkimMergeData = AlcaSkimMergeByAlcaSkimIdsDP(alcaSkimIds, params.orderby, params.asc, params.page)
            alcaMergeFiles = getTableGui(T_ALCA_MERGE_FILES, alcaSkimMergeData)
        except:
            alcaMergeFiles = formatExceptionInfo()


        return self.templatePage (
                "history", {
                    T_STREAM_FILES: streamerFiles,
                    T_REPACK_JOBS : repackJobs,
                    T_REPACK_FILES : repackFiles,
                    T_MERGE_JOBS : mergeJobs,
                    T_MERGE_FILES : mergeFiles,
                    T_RECO_JOBS : recoJobs,
                    T_RECO_FILES : recoFiles,
                    T_RECO_MERGE_JOBS : recoMergeJobs,
                    T_RECO_MERGE_FILES : recoMergeFiles,
                    T_ALCA_JOBS : alcaJobs,
                    T_ALCA_FILES : alcaFiles,
                    T_ALCA_MERGE_JOBS : alcaMergeJobs,
                    T_ALCA_MERGE_FILES : alcaMergeFiles,
                    'entity' : entity + ' ' + str(id),
                    "tmplPath" : TMPL_PATH
                }
            )

    @expose
    def jobs(self, **kwds):
        """
        Display jobs for particular run and/or status
        """
        globals.pageParams = kwds
        if globals.pageParams.has_key('table'):
            return self.jobsRepack( **kwds )
        else:
            if globals.pageParams.has_key('inputTier'):
                return self.jobsWMBS( **kwds )
            else: return "<br />No input tier or table name specified"

    def jobsWMBS( self, **kwds ):
        """
        Display information on WMBS jobs
        """
        globals.pageParams = kwds
        error = ""
        inputTier = str(kwds['inputTier'])
        if globals.pageParams.has_key('mergeFlag'):
            mergeFlag = kwds['mergeFlag']
        else: error += "<br />No status flag specified"
        if globals.pageParams.has_key('status'):
            status = kwds['status']
        else: error += "<br />No status flag specified"
        if globals.pageParams.has_key('runid'):
            try:
                runid = int(kwds['runid'])
            except ValueError:
                error += "<br />Run id is not an int"
        else: runid = None

        params = self._getTableParams(T_JOBS)

        if error=="":
            if runid != None:
                jtd = WMBSJobByStatusAndRunIdDP(inputTier, mergeFlag, status, runid, params.orderby, params.asc, params.page)
            else:
                jtd = WMBSJobByStatusDP(inputTier, mergeFlag, status, params.orderby, params.asc, params.page)
            jt = getTableGui(T_JOBS, jtd)
        else:
            jt = error

        return self.templatePage("jobs", {
            T_JOBS: jt,
            "time" : ctime(),
            "tmplPath" : TMPL_PATH
        })

    def jobsRepack( self, **kwds):
        globals.pageParams = kwds
        error = ""
        table = str(kwds['table'])
        if globals.pageParams.has_key('status'):
            try:
                status = int(kwds['status'])
            except ValueError:
                error += "<br />Status is not an int"
        else: error += "<br />No status flag specified"
        if globals.pageParams.has_key('runid'):
            try:
                runid = int(kwds['runid'])
            except ValueError:
                error += "<br />Run id is not an int"
        else: runid = None

        params = self._getTableParams(T_JOBS)

        if table != "" and error == "":
            if table == TABLES.REPACK_JOBS:
                if runid != None:
                    jtd = RepackJobByStatusAndRunIdDP(status, runid, params.orderby, params.asc, params.page)
                else:
                    jtd = RepackJobByStatusDP(status, params.orderby, params.asc, params.page)
            else: error += "<br />Bad jobs table name specified"

        if error == "":
            jt = getTableGui(T_JOBS, jtd)
        else:
            jt = error

        return self.templatePage("jobs", {
            T_JOBS: jt,
            "time" : ctime(),
            "tmplPath" : TMPL_PATH
        })

    @expose
    def run(self, runid, **kwds):
        """
        Display run statistics
        """
        globals.pageParams = kwds

        try:
            runid = int(runid)
        except ValueError:
            return self.templatePage ("error", {
                'msg': formatExceptionInfo(),
                "tmplPath" : TMPL_PATH
            })

        params = self._getTableParams(RUN_REPACK_MERGE_T)

        try:
            rrmtd = RepackMergeByRunIdDP(runid, params.orderby, params.asc, params.page)
            rrmt = getTableGui(RUN_REPACK_MERGE_T, rrmtd)
        except: rrmt = formatExceptionInfo()

        params = self._getTableParams(RUN_REPACK_T)

        try:
            rrtd = RepackByRunIdDP(runid, params.orderby, params.asc, params.page)
            rrt = getTableGui(RUN_REPACK_T, rrtd)
        except: rrt = formatExceptionInfo()

        params = self._getTableParams(RUN_RECO_T)

        try:
            rrtd = RecoByRunIdDP(runid, params.orderby, params.asc, params.page)
            rret = getTableGui(RUN_RECO_T, rrtd)
        except: rret = formatExceptionInfo()

        params = self._getTableParams(RUN_STREAMER_T)

        try:
            rstd = StreamerByRunIdDP(runid, params.orderby, params.asc, params.page)
            rst = getTableGui(RUN_STREAMER_T, rstd)
        except: rst = formatExceptionInfo()

	params = self._getTableParams(RUN_RECO_MERGE_T)

        try:
            rrecomtd = RecoMergeByRunIdDP(runid, params.orderby, params.asc, params.page)
            rrecomt = getTableGui(RUN_RECO_MERGE_T, rrecomtd)
        except: rrecomt = formatExceptionInfo()

        params = self._getTableParams(RUN_ALCASKIM_T)

        try:
            ratd = AlcaskimByRunIdDP(runid, params.orderby, params.asc, params.page)
            rat = getTableGui(RUN_ALCASKIM_T, ratd)
        except: rat = formatExceptionInfo()

        params = self._getTableParams(RUN_ALCASKIM_MERGE_T)

        try:
            ramtd = AlcaskimMergeByRunIdDP(runid, params.orderby, params.asc, params.page)
            ramt = getTableGui(RUN_ALCASKIM_MERGE_T, ramtd)
        except: ramt = formatExceptionInfo()

        try:
            rrstd = RepackStatsDataProvider(runid)
            rrst = getVertTableGui(RUN_REPACK_T, rrstd)
        except: rrst = formatExceptionInfo()

        try:
            rmstd = JobsStatsDataProvider("RAW", "Repack Merge stats", runid, mergeJobs=True)
	    rmstd.load()
            rmst = getVertTableGui(RUN_MERGE_STATS_T, rmstd)
        except: rmst = formatExceptionInfo()

        try:
            rcstd = JobsStatsDataProvider("RAW", "Reco stats", runid)
	    rcstd.load()
            rcst = getVertTableGui(RUN_RECO_STATS_T, rcstd)
        except: rcst = formatExceptionInfo()

        try:
            rrmstd = JobsStatsDataProvider("RECO", "Reco Merge stats", runid,  mergeJobs=True)
            rrmstd.load()
            rrmst = getVertTableGui(RUN_RECO_MERGE_STATS_T, rrmstd)
        except: rrmst = formatExceptionInfo()

        try:
            rastd = JobsStatsDataProvider("RECO", "AlcaSkim stats", runid)
            rastd.load()
            rast = getVertTableGui(RUN_ALCASKIM_STATS_T, rastd)
        except: rast = formatExceptionInfo()

        try:
            rasmtd = JobsStatsDataProvider("ALCARECO", "AlcaSkim Merge stats", runid, mergeJobs=True)
            rasmtd.load()
            rasmt = getVertTableGui(RUN_ALCASKIM_MERGE_STATS_T, rasmtd)
        except: rasmt = formatExceptionInfo()

        try:
            rdtd = RunConfigKeyProvider(runid)
            rdt = getDoubleTableGui(RUN_CONFIG_KEY_T, rdtd)
        except: rdt = formatExceptionInfo()

        return self.templatePage("run", {
            RUN_REPACK_T : rrt,
            RUN_RECO_T : rret,
            RUN_RECO_MERGE_T : rrecomt,
            RUN_ALCASKIM_T : rat,
            RUN_ALCASKIM_MERGE_T : ramt,
            RUN_STREAMER_T : rst,
            RUN_REPACK_STATS_T : rrst,
            RUN_MERGE_STATS_T : rmst,
            RUN_RECO_STATS_T : rcst,
	    RUN_RECO_MERGE_STATS_T : rrmst, 
	    RUN_ALCASKIM_STATS_T : rast,
	    RUN_ALCASKIM_MERGE_STATS_T : rasmt,
            RUN_REPACK_MERGE_T : rrmt,
            RUN_CONFIG_KEY_T : rdt,
            #"streamRepackG" : srg,
            "runid" : runid,
            "time" : ctime(),
            "tmplPath" : TMPL_PATH
        })

    @expose
    def index(self, **kwds):
        """
        Display last runs, job statistics and running jobs
        """

        globals.pageParams = kwds

        #### Runs table ####

        params = self._getTableParams(RUNS_T)

        try:
            runsTableTd = RunsDataProvider(params.orderby, params.asc, params.page)
            runsTable = getTableGui(RUNS_T, runsTableTd, 100)
        except: runsTable = formatExceptionInfo()

        #### Repacking status table ####

        params = self._getTableParams(REPACKER_T)

        try:
            repackerTd = RepackRunningJobDP(params.orderby, params.asc, params.page)
            repackerTable = getTableGui(REPACKER_T, repackerTd)
        except: repackerTable = formatExceptionInfo()

        #### Reconstruction status table ####

        params = self._getTableParams(RECO_T)

        try:
            recoTd = WMBSJobByStatusDP("RAW","NOT","ACQUIRED",params.orderby, params.asc, params.page)
            recoTable = getTableGui(RECO_T, recoTd)
        except: recoTable = formatExceptionInfo()

        params = self._getTableParams(RECO_MERGE_T)

        try:
            recoMergeTd = WMBSJobByStatusDP("RECO","","ACQUIRED",params.orderby, params.asc, params.page)
            recoMergeTable = getTableGui(RECO_MERGE_T, recoMergeTd)
        except: recoMergeTable = formatExceptionInfo()

        #### Merging status table ####

        params = self._getTableParams(MERGE_T)

        try:
            mergeTd = WMBSJobByStatusDP( "RAW", "", "ACQUIRED", params.orderby, params.asc, params.page )
            mergeTable = getTableGui(MERGE_T, mergeTd)
        except: mergeTable = formatExceptionInfo()

        #### Alcaskim status tables ####

        params = self._getTableParams(ALCASKIM_T)

        try:
            alcaskimTd = WMBSJobByStatusDP("RECO","NOT","ACQUIRED",params.orderby, params.asc, params.page)
            alcaskimTable = getTableGui(ALCASKIM_T, alcaskimTd)
        except: alcaskimTable = formatExceptionInfo()

        params = self._getTableParams(ALCASKIM_MERGE_T)

        try:
            alcaskimMergeTd = WMBSJobByStatusDP("ALCARECO","","ACQUIRED",params.orderby, params.asc, params.page)
            alcaskimMergeTable = getTableGui(ALCASKIM_MERGE_T, alcaskimMergeTd)
        except: alcaskimMergeTable = formatExceptionInfo()

        #### Job status tables ####

        try:
            repackStatsTd = StatsDataProvider("Repack stats", TABLES.REPACK_JOBS)
            repackStatsTd.load()
            repackStatsTable = getVertTableGui(REPACK_S_T, repackStatsTd)
        except: repackStatsTable = formatExceptionInfo()

        try:
	    mergeStatsTd = JobsStatsDataProvider("RAW", "Merge stats", mergeJobs=True)
            mergeStatsTd.load()
            mergeStatsTable = getVertTableGui(MERGE_S_T, mergeStatsTd)
        except: mergeStatsTable = formatExceptionInfo()

        try:
            recoStatsTd = JobsStatsDataProvider("RAW", "Reco stats")
            recoStatsTd.load()
            recoStatsTable = getVertTableGui(RECO_S_T, recoStatsTd)
        except: recoStatsTable = formatExceptionInfo()

        try:
            recoMergeStatsTd = JobsStatsDataProvider("RECO", "Reco Merge stats", mergeJobs=True)
            recoMergeStatsTd.load()
            recoMergeStatsTable = getVertTableGui(RECO_MERGE_S_T, recoMergeStatsTd)
        except: recoMergeStatsTable = formatExceptionInfo()

        try:
            alcaskimStatsTd = JobsStatsDataProvider("RECO", "AlcaSkim stats")
            alcaskimStatsTd.load()
            alcaskimStatsTable = getVertTableGui(ALCASKIM_S_T, alcaskimStatsTd)
        except: alcaskimStatsTable = formatExceptionInfo()

        try:
            alcaskimMergeStatsTd = JobsStatsDataProvider("ALCARECO", "AlcaSkim Merge stats", mergeJobs=True)
            alcaskimMergeStatsTd.load()
            alcaskimMergeStatsTable = getVertTableGui(ALCASKIM_MERGE_S_T, alcaskimMergeStatsTd)
        except: alcaskimMergeStatsTable = formatExceptionInfo()

        url = request.base
        if DEV_SERVER in url: dev = True
        else: dev = False

        return self.templatePage("index", {
            RUNS_T : runsTable,
            REPACKER_T : repackerTable,
            RECO_T : recoTable,
            RECO_MERGE_T : recoMergeTable,
	    ALCASKIM_T : alcaskimTable, 
	    ALCASKIM_MERGE_T : alcaskimMergeTable, 
            MERGE_T : mergeTable,
            MERGE_S_T : mergeStatsTable,
            REPACK_S_T : repackStatsTable,
            RECO_S_T : recoStatsTable,
            RECO_MERGE_S_T : recoMergeStatsTable,
	    ALCASKIM_S_T : alcaskimStatsTable,
	    ALCASKIM_MERGE_S_T : alcaskimMergeStatsTable,
            "time" : ctime(),
            "dev" : dev,
            "tmplPath" : TMPL_PATH
        })

    #### Private ####

    def _getTableParams(self, tableid, orderby = 0, asc = False, page = 0):
        """
        Parse table params from URL
        """
        if globals.pageParams.has_key(tableid + "_p"):
            try:
                pages = globals.pageParams[tableid + "_p"]
                if pages == "all": page = None
                else:
                    page = int(globals.pageParams[tableid + "_p"])
                    if page < 0:
                        page = 0
                        self.context.Logger().error("Page num < 0")
            except ValueError:
                self.context.Logger().error("Could not convert page num to an int")

        if globals.pageParams.has_key(tableid):
            words = globals.pageParams[tableid].rsplit("_")
            try:
                orderby = int(words[0])
                if orderby < 0:
                    orderby = 0
                    self.context.Logger().error("Col id < 0")

            except ValueError:
                self.context.Logger().error("Could not convert table row id to an int")

            if (len(words) == 2):
                if (words[1] == "True"):
                    asc = True
                else: asc = False
            else:
                self.context.Logger().error("Badly formed table params")

        params = TableParams(orderby, asc, page)

        return params

class TableParams:
    """
    Table parameters
    """
    def __init__(self, orderby = 0, asc = False, page = 0):
        """
        Parameters:

        orderby    table ordering column id
        asc        order ascending or descending?
        page       page number to display
        """
        self.orderby = orderby
        self.asc = asc
        self.page = page

# Declare controllers
DeclarePlugin ("/Controllers/" + PROJECT_NAME + "/" + PROJECT_NAME, T0Mon, {"baseUrl": "/" + PROJECT_NAME})

