from sqlalchemy import *

from dataProviderBase import *
from T0Mon.utils import *
from T0Mon.consts import *
from T0Mon.sqlManager import *

from T0Mon.globals import logger
import T0Mon.config

### Module for classes providing data about T0 files ###

class BaseDP(PagedTableDataProvider):
    cols = Cols([
        Col("Id"),
        Col("Filesize", decorator = humanFilesize),
        Col("Events"),
#        Col("Status"),            # TODO : check these columns in the database, add to the final version of this table
#        Col("Export status"),
        Col("Dataset"),
        Col("Lfn", decorator = lambda x: textToImageAlt(x))
    ])

    def __init__(self, type, orderby = 0, asc = True, page = 0):
	self.type = type
        PagedTableDataProvider.__init__(self, orderby, asc, page)
	self.selectCond = """ """
	self.fromCond = """ """
	self.whereCond = """ """

    def _buildCountQuery(self):
        wmbsFileDetailsTable = Tbl(TABLES.WMBS_FILE_DETAILS, True)
        wmbsDatasetAssocTable = Tbl(TABLES.WMBS_FILE_DATASET_PATH_ASSOC, True)
        datasetTable = Tbl(TABLES.DATASET_PATH, True)
        dataTierTable = Tbl(TABLES.DATA_TIER, True)

        self.countQuery = """ SELECT COUNT(DISTINCT f.id)
                                FROM """ + wmbsFileDetailsTable + """ f INNER JOIN """ + wmbsDatasetAssocTable  + """ a ON (a.file_id = f.id)
                                        INNER JOIN """ + datasetTable  + """ dp ON (a.dataset_path_id = dp.id)
                                        INNER JOIN """ + dataTierTable + """ dt ON (dp.data_tier = dt.id)
                                         """ + self.fromCond + """
                                WHERE dt.name = '""" + self.type + """' """ + self.whereCond + """ """

    def _buildMainQuery(self):
        wmbsFileDetailsTable = Tbl(TABLES.WMBS_FILE_DETAILS, True)
        wmbsDatasetAssocTable = Tbl(TABLES.WMBS_FILE_DATASET_PATH_ASSOC, True)
        datasetTable = Tbl(TABLES.DATASET_PATH, True)
        dataTierTable = Tbl(TABLES.DATA_TIER, True)
        primaryDatasetTable = Tbl(TABLES.PRIMARY_DATASET, True)

        if self.asc:
                order = "ASC"
        else:
                order = "DESC"

        self.mainQuery = """ SELECT DISTINCT f.id, f.filesize, f.events, pd.name, f.lfn""" + self.selectCond + """ 
                                FROM """ + wmbsFileDetailsTable + """ f INNER JOIN """ + wmbsDatasetAssocTable  + """ a ON (a.file_id = f.id)
                                        INNER JOIN """ + datasetTable  + """ dp ON (a.dataset_path_id = dp.id)
                                        INNER JOIN """ + dataTierTable + """ dt ON (dp.data_tier = dt.id)
                                        INNER JOIN """ + primaryDatasetTable + """ pd ON (dp.primary_dataset = pd.id)
                                         """ + self.fromCond + """
                                WHERE dt.name = '""" + self.type + """' """ + self.whereCond + """
                                ORDER BY  """  + str(int(self.orderby)+1) + """ """ + order


class BaseRepackDP(BaseDP):
    def __init__(self, orderby = 0, asc = True, page = 0):
        BaseDP.__init__(self, 'RAW', orderby, asc, page)
        self.title = "Repacked files"
	self.cols["Id"].decorator = buildRepackLink


class RepackByIdsDP(BaseRepackDP):
    def __init__(self, ids, orderby = 0, asc = True, page = 0):
        BaseRepackDP.__init__(self, orderby, asc, page)

        ids_str = ""
        numIds = len(ids)
        if numIds != 0:
                if numIds > 1:
                        for i in range(numIds-1):
                                ids_str += str(ids[i]) + ", "
                ids_str += str(ids[-1])

        wmbsFileRunlumiMapTable = Tbl(TABLES.WMBS_FILE_RUNLUMI_MAP, True)
        wmbsFilesetFilesTable = Tbl(TABLES.WMBS_FILESET_FILES, True)
        wmbsFilesetTable = Tbl(TABLES.WMBS_FILESET, True)

	self.whereCond = """ and fs.name = 'Mergeable' and f.id IN (""" + ids_str + """)"""

        self.fromCond = """ INNER JOIN """ + wmbsFileRunlumiMapTable + """ m ON (f.id = m.fileid)
                            INNER JOIN """ + wmbsFilesetFilesTable + """ fsf ON (f.id = fsf.fileid)
                            INNER JOIN """ + wmbsFilesetTable + """ fs ON (fs.id = fsf.fileset) """

        if numIds != 0:
                BaseRepackDP._load(self)

class RepackByRunIdDP(BaseRepackDP):
    def __init__(self, runid, orderby = 0, asc = True, page = 0):
        BaseRepackDP.__init__(self, orderby, asc, page)
        self.title = "Temporary Repacked files"

        wmbsFileRunlumiMapTable = Tbl(TABLES.WMBS_FILE_RUNLUMI_MAP, True)
	wmbsFilesetFilesTable = Tbl(TABLES.WMBS_FILESET_FILES, True)
	wmbsFilesetTable = Tbl(TABLES.WMBS_FILESET, True)

	self.whereCond = """ and m.run = """ + str(runid) + """ and fs.name = 'Mergeable' """
	
	self.fromCond = """ INNER JOIN """ + wmbsFileRunlumiMapTable + """ m ON (f.id = m.fileid) 
			    INNER JOIN """ + wmbsFilesetFilesTable + """ fsf ON (f.id = fsf.fileid)
			    INNER JOIN """ + wmbsFilesetTable + """ fs ON (fs.id = fsf.fileset) """

        BaseRepackDP._load(self)

class RepackByRepackMergeIdsDP(BaseRepackDP): 
    def __init__(self, ids, orderby = 0, asc = True, page = 0):
        BaseRepackDP.__init__(self, orderby, asc, page)
	self.title = "Temporary Repacked files"

        wmbsFilesetFilesTable = Tbl(TABLES.WMBS_FILESET_FILES, True)
        wmbsFilesetTable = Tbl(TABLES.WMBS_FILESET, True)
        wmbsFileParentTable = Tbl(TABLES.WMBS_FILE_PARENT, True)

        ids_str = ""
        numIds = len(ids)
        if numIds != 0:
                if numIds > 1:
                        for i in range(numIds-1):
                                ids_str += str(ids[i]) + ", "
                ids_str += str(ids[-1])

        self.whereCond = """ and fs.name = 'Mergeable' and fp.child IN (""" + ids_str + """) """

        self.fromCond = """ INNER JOIN """ + wmbsFilesetFilesTable + """ fsf ON (f.id = fsf.fileid)
                            INNER JOIN """ + wmbsFilesetTable + """ fs ON (fs.id = fsf.fileset)
                            INNER JOIN """ + wmbsFileParentTable + """ fp ON (f.id = fp.parent) """

        if len(ids) != 0:
            BaseRepackDP._load(self)


class RepackByRepackMergeJobIdsDP(BaseRepackDP):
    def __init__(self, ids, orderby = 0, asc = True, page = 0):
        BaseRepackDP.__init__(self, orderby, asc, page)
        self.title = "Temporary Repacked files"

        ids_str = ""
        numIds = len(ids)
        if numIds != 0:
                if numIds > 1:
                        for i in range(numIds-1):
                                ids_str += str(ids[i]) + ", "
                ids_str += str(ids[-1])

        wmbsJobAssocTable = Tbl(TABLES.WMBS_JOB_ASSOC, True)

        self.whereCond = """ and ja.job IN (""" + ids_str + """) """

        self.fromCond = """ INNER JOIN """ + wmbsJobAssocTable + """ ja ON (ja.fileid = f.id) """

        if numIds != 0:
                BaseRepackDP._load(self)


class RepackByStreamerIdsDP(BaseRepackDP): 
    def __init__(self, ids, orderby = 0, asc = True, page = 0):
        BaseRepackDP.__init__(self, orderby, asc, page)
	self.title = "Temporary Repacked files"

	repackStreamerAssocTable = Tbl(TABLES.REPACK_STREAMER_ASSOC, True)

	ids_str = ""
	numIds = len(ids)
	if numIds != 0:
		if numIds > 1:
			for i in range(numIds-1):
				ids_str += str(ids[i]) + ", "
		ids_str += str(ids[-1])

        self.whereCond = """ and a.streamer_id IN (""" + ids_str + """) """
	
	self.fromCond = """ INNER JOIN """ + repackStreamerAssocTable + """ a ON (a.repacked_id = f.id) """

	if numIds != 0:
	        BaseRepackDP._load(self)

class BaseRepackMergeDP(BaseRepackDP):
    def __init__(self, orderby = 0, asc = True, page = 0):
        BaseRepackDP.__init__(self, orderby, asc, page)
        self.title = "Repacked files"
	self.cols["Id"].decorator = buildRepackMergeLink


class RepackMergeByRecoIdsDP(BaseRepackMergeDP):
    def __init__(self, ids, orderby = 0, asc = True, page = 0):
        BaseRepackMergeDP.__init__(self, orderby, asc, page)

        wmbsFilesetFilesTable = Tbl(TABLES.WMBS_FILESET_FILES, True)
        wmbsFilesetTable = Tbl(TABLES.WMBS_FILESET, True)
        wmbsFileParentTable = Tbl(TABLES.WMBS_FILE_PARENT, True)

        ids_str = ""
        numIds = len(ids)
        if numIds != 0:
                if numIds > 1:
                        for i in range(numIds-1):
                                ids_str += str(ids[i]) + ", "
                ids_str += str(ids[-1])

        self.whereCond = """ and fs.name = 'DBSUploadable' and fp.child IN (""" + ids_str + """) """

        self.fromCond = """ INNER JOIN """ + wmbsFilesetFilesTable + """ fsf ON (f.id = fsf.fileid)
                            INNER JOIN """ + wmbsFilesetTable + """ fs ON (fs.id = fsf.fileset)
                            INNER JOIN """ + wmbsFileParentTable + """ fp ON (f.id = fp.parent) """

        if len(ids) != 0:
	        BaseRepackMergeDP._load(self)

class RepackMergeByRecoJobIdsDP(BaseRepackMergeDP):
    def __init__(self, ids, orderby = 0, asc = True, page = 0):
        BaseRepackMergeDP.__init__(self, orderby, asc, page)

        ids_str = ""
        numIds = len(ids)
        if numIds != 0:
                if numIds > 1:
                        for i in range(numIds-1):
                                ids_str += str(ids[i]) + ", "
                ids_str += str(ids[-1])

        wmbsJobAssocTable = Tbl(TABLES.WMBS_JOB_ASSOC, True)
        wmbsFilesetFilesTable = Tbl(TABLES.WMBS_FILESET_FILES, True)
        wmbsFilesetTable = Tbl(TABLES.WMBS_FILESET, True)

        self.whereCond = """ and fs.name = 'DBSUploadable' and ja.job IN (""" + ids_str + """) """

        self.fromCond = """ INNER JOIN """ + wmbsFilesetFilesTable + """ fsf ON (f.id = fsf.fileid)
                            INNER JOIN """ + wmbsFilesetTable + """ fs ON (fs.id = fsf.fileset)
			    INNER JOIN """ + wmbsJobAssocTable + """ ja ON (ja.fileid = f.id) """

        if numIds != 0:
                BaseRepackDP._load(self)


class RepackMergeByIdsDP(BaseRepackMergeDP):
    def __init__(self, ids, orderby = 0, asc = True, page = 0):
        BaseRepackMergeDP.__init__(self, orderby, asc, page)

        wmbsFilesetFilesTable = Tbl(TABLES.WMBS_FILESET_FILES, True)
        wmbsFilesetTable = Tbl(TABLES.WMBS_FILESET, True)

        ids_str = ""
        numIds = len(ids)
        if numIds != 0:
                if numIds > 1:
                        for i in range(numIds-1):
                                ids_str += str(ids[i]) + ", "
                ids_str += str(ids[-1])

        self.whereCond = """ and fs.name = 'DBSUploadable' and f.id IN (""" + ids_str + """) """

        self.fromCond = """ INNER JOIN """ + wmbsFilesetFilesTable + """ fsf ON (f.id = fsf.fileid)
                            INNER JOIN """ + wmbsFilesetTable + """ fs ON (fs.id = fsf.fileset) """

        BaseRepackMergeDP._load(self)

class RepackMergeByRepackMergeJobIdsDP(BaseRepackMergeDP):
    def __init__(self, ids, orderby = 0, asc = True, page = 0):
        BaseRepackMergeDP.__init__(self, orderby, asc, page)
        repackedTable = Tbl(TABLES.REPACKED)
        assocTable = Tbl(TABLES.MERGE_JOB_REPACK_ASSOC)
        parentageTable = Tbl(TABLES.REPACKED_MERGE_PARENTAGE)
        self.whereCond = repackedTable.c.repacked_id.in_(
            select([parentageTable.c.output_id], parentageTable.c.input_id.in_(
                select([assocTable.c.repacked_id], assocTable.c.job_id.in_(ids))
            ))
        )
        BaseRepackMergeDP._load(self)

class RepackMergeByRepackIdsDP(BaseRepackMergeDP):
    def __init__(self, ids, orderby = 0, asc = True, page = 0):
        BaseRepackMergeDP.__init__(self, orderby, asc, page)

        wmbsFilesetFilesTable = Tbl(TABLES.WMBS_FILESET_FILES, True)
        wmbsFilesetTable = Tbl(TABLES.WMBS_FILESET, True)
	wmbsFileParentTable = Tbl(TABLES.WMBS_FILE_PARENT, True)

        ids_str = ""
        numIds = len(ids)
        if numIds != 0:
                if numIds > 1:
                        for i in range(numIds-1):
                                ids_str += str(ids[i]) + ", "
                ids_str += str(ids[-1])

        self.whereCond = """ and fs.name = 'DBSUploadable' and fp.parent IN (""" + ids_str + """) """

        self.fromCond = """ INNER JOIN """ + wmbsFilesetFilesTable + """ fsf ON (f.id = fsf.fileid)
                            INNER JOIN """ + wmbsFilesetTable + """ fs ON (fs.id = fsf.fileset)
			    INNER JOIN """ + wmbsFileParentTable + """ fp ON (f.id = fp.child) """

        if len(ids) != 0:
            BaseRepackMergeDP._load(self)

class RepackMergeByRunIdDP(BaseRepackMergeDP):
    def __init__(self, runid, orderby = 0, asc = True, page = 0):
        BaseRepackMergeDP.__init__(self, orderby, asc, page)

        wmbsFileRunlumiMapTable = Tbl(TABLES.WMBS_FILE_RUNLUMI_MAP, True)
        wmbsFilesetFilesTable = Tbl(TABLES.WMBS_FILESET_FILES, True)
        wmbsFilesetTable = Tbl(TABLES.WMBS_FILESET, True)

        self.whereCond = """ and m.run = """ + str(runid) + """ and fs.name = 'DBSUploadable' """

        self.fromCond = """ INNER JOIN """ + wmbsFileRunlumiMapTable + """ m ON (f.id = m.fileid)
	                    INNER JOIN """ + wmbsFilesetFilesTable + """ fsf ON (f.id = fsf.fileid)
            	            INNER JOIN """ + wmbsFilesetTable + """ fs ON (fs.id = fsf.fileset) """

        BaseRepackMergeDP._load(self)

class BaseRecoDP(BaseDP):
    def __init__(self, orderby = 0, asc = True, page = 0):
        BaseDP.__init__(self, 'RECO', orderby, asc, page)
        self.title = "Reco files"
	self.cols["Id"].decorator = buildRecoLink

class RecoMergeByRunIdDP(BaseRecoDP):
        def __init__(self, runid, orderby = 0, asc = True, page = 0):
                BaseRecoDP.__init__(self, orderby, asc, page)
		self.cols["Id"].decorator = buildRecoMergeLink

                wmbsFileRunlumiMapTable = Tbl(TABLES.WMBS_FILE_RUNLUMI_MAP, True)
                wmbsFilesetFilesTable = Tbl(TABLES.WMBS_FILESET_FILES, True)
                wmbsFilesetTable = Tbl(TABLES.WMBS_FILESET, True)

                self.whereCond = """ and m.run = """ + str(runid) + """ and fs.name = 'DBSUploadable' """

                self.fromCond = """ INNER JOIN """ + wmbsFileRunlumiMapTable + """ m ON (f.id = m.fileid)
		                    INNER JOIN """ + wmbsFilesetFilesTable + """ fsf ON (f.id = fsf.fileid)
			            INNER JOIN """ + wmbsFilesetTable + """ fs ON (fs.id = fsf.fileset) """

                BaseRecoDP._load(self)

class RecoMergeByIdsDP(BaseRecoDP):
    def __init__(self, ids, orderby = 0, asc = True, page = 0):
        BaseRecoDP.__init__(self, orderby, asc, page)
	self.cols["Id"].decorator = buildRecoMergeLink

        wmbsFilesetFilesTable = Tbl(TABLES.WMBS_FILESET_FILES, True)
        wmbsFilesetTable = Tbl(TABLES.WMBS_FILESET, True)

        ids_str = ""
        numIds = len(ids)
        if numIds != 0:
                if numIds > 1:
                        for i in range(numIds-1):
                                ids_str += str(ids[i]) + ", "
                ids_str += str(ids[-1])

        self.whereCond = """ and fs.name = 'DBSUploadable' and f.id IN (""" + ids_str + """) """

        self.fromCond = """ INNER JOIN """ + wmbsFilesetFilesTable + """ fsf ON (f.id = fsf.fileid)
                            INNER JOIN """ + wmbsFilesetTable + """ fs ON (fs.id = fsf.fileset) """

        BaseRecoDP._load(self)


class RecoMergeByRecoIdsDP(BaseRecoDP):
    def __init__(self, ids, orderby = 0, asc = True, page = 0):
        BaseRecoDP.__init__(self, orderby, asc, page)
	self.cols["Id"].decorator = buildRecoMergeLink

        wmbsFilesetFilesTable = Tbl(TABLES.WMBS_FILESET_FILES, True)
        wmbsFilesetTable = Tbl(TABLES.WMBS_FILESET, True)
        wmbsFileParentTable = Tbl(TABLES.WMBS_FILE_PARENT, True)

        ids_str = ""
        numIds = len(ids)
        if numIds != 0:
                if numIds > 1:
                        for i in range(numIds-1):
                                ids_str += str(ids[i]) + ", "
                ids_str += str(ids[-1])

        self.whereCond = """ and fs.name = 'DBSUploadable' and fp.parent IN (""" + ids_str + """) """

        self.fromCond = """ INNER JOIN """ + wmbsFilesetFilesTable + """ fsf ON (f.id = fsf.fileid)
                            INNER JOIN """ + wmbsFilesetTable + """ fs ON (fs.id = fsf.fileset)
                            INNER JOIN """ + wmbsFileParentTable + """ fp ON (f.id = fp.child) """

        if len(ids) != 0:
            BaseRecoDP._load(self)

class RecoMergeByAlcaSkimJobIdsDP(BaseRecoDP):
    def __init__(self, ids, orderby = 0, asc = True, page = 0):
        BaseRecoDP.__init__(self, orderby, asc, page)
        self.cols["Id"].decorator = buildRecoMergeLink

        wmbsJobAssocTable = Tbl(TABLES.WMBS_JOB_ASSOC, True)
        wmbsFilesetFilesTable = Tbl(TABLES.WMBS_FILESET_FILES, True)
        wmbsFilesetTable = Tbl(TABLES.WMBS_FILESET, True)

        ids_str = ""
        numIds = len(ids)
        if numIds != 0:
                if numIds > 1:
                        for i in range(numIds-1):
                                ids_str += str(ids[i]) + ", "
                ids_str += str(ids[-1])

        self.whereCond = """ and fs.name = 'DBSUploadable' and ja.job IN (""" + ids_str + """) """

        self.fromCond = """ INNER JOIN """ + wmbsFilesetFilesTable + """ fsf ON (f.id = fsf.fileid)
                            INNER JOIN """ + wmbsFilesetTable + """ fs ON (fs.id = fsf.fileset)
                            INNER JOIN """ + wmbsJobAssocTable + """ ja ON (ja.fileid = f.id) """

        if len(ids) != 0:
            BaseRecoDP._load(self)

class RecoMergeByAlcaSkimIdsDP(BaseRecoDP): 
    def __init__(self, ids, orderby = 0, asc = True, page = 0):
        BaseRecoDP.__init__(self, orderby, asc, page)
	self.cols["Id"].decorator = buildRecoMergeLink

        wmbsFilesetFilesTable = Tbl(TABLES.WMBS_FILESET_FILES, True)
        wmbsFilesetTable = Tbl(TABLES.WMBS_FILESET, True)
        wmbsFileParentTable = Tbl(TABLES.WMBS_FILE_PARENT, True)

        ids_str = ""
        numIds = len(ids)
        if numIds != 0:
                if numIds > 1:
                        for i in range(numIds-1):
                                ids_str += str(ids[i]) + ", "
                ids_str += str(ids[-1])

        self.whereCond = """ and fs.name = 'DBSUploadable' and fp.child IN (""" + ids_str + """) """

        self.fromCond = """ INNER JOIN """ + wmbsFilesetFilesTable + """ fsf ON (f.id = fsf.fileid)
                            INNER JOIN """ + wmbsFilesetTable + """ fs ON (fs.id = fsf.fileset)
                            INNER JOIN """ + wmbsFileParentTable + """ fp ON (f.id = fp.parent) """

        if len(ids) != 0:
            BaseRecoDP._load(self)


class BaseAlcaskimDP(BaseDP):
    cols = Cols([
        Col("Id"),
        Col("Filesize", decorator = humanFilesize),
        Col("Events"),
#        Col("Status"),            # TODO : check these columns in the database, add to the final version of this table
#        Col("Export status"),
        Col("Dataset"),
        Col("Lfn", decorator = lambda x: textToImageAlt(x)),
	Col("Skim")
    ])
    
    def __init__(self, orderby = 0, asc = True, page = 0):
        BaseDP.__init__(self, 'ALCARECO', orderby, asc, page)
	self.cols["Id"].decorator = buildAlcaSkimLink
        self.title = "AlcaSkim files"

    def _analysis(self):
        for line in self.data:
            processedDatasetName = line[self.cols.getId("Skim")]
            processedDatasetSkimName = processedDatasetName.split('-')[1]
            line[self.cols.getId("Skim")] = processedDatasetSkimName


class AlcaskimByRunIdDP(BaseAlcaskimDP):
        def __init__(self, runid, orderby = 0, asc = True, page = 0):
                BaseAlcaskimDP.__init__(self, orderby, asc, page)
	        self.title = "Temporary AlcaSkim files"

	        wmbsFileRunlumiMapTable = Tbl(TABLES.WMBS_FILE_RUNLUMI_MAP, True)
        	wmbsFilesetFilesTable = Tbl(TABLES.WMBS_FILESET_FILES, True)
	        wmbsFilesetTable = Tbl(TABLES.WMBS_FILESET, True)
                processedDatasetTable = Tbl(TABLES.PROCESSED_DATASET, True)

		self.selectCond = """, proc.name as skim"""

        	self.whereCond = """ and m.run = """ + str(runid) + """ and fs.name = 'Mergeable' """

	        self.fromCond = """ INNER JOIN """ + wmbsFileRunlumiMapTable + """ m ON (f.id = m.fileid)
        	                    INNER JOIN """ + wmbsFilesetFilesTable + """ fsf ON (f.id = fsf.fileid)
                	            INNER JOIN """ + wmbsFilesetTable + """ fs ON (fs.id = fsf.fileset)
				    INNER JOIN """ + processedDatasetTable + """ proc ON (dp.processed_dataset = proc.id) """

                BaseAlcaskimDP._load(self)


class AlcaskimMergeByRunIdDP(BaseAlcaskimDP):
        def __init__(self, runid, orderby = 0, asc = True, page = 0):
                BaseAlcaskimDP.__init__(self, orderby, asc, page)
		self.cols["Id"].decorator = buildAlcaSkimMergeLink
                self.title = "AlcaSkim files"

                wmbsFileRunlumiMapTable = Tbl(TABLES.WMBS_FILE_RUNLUMI_MAP, True)
                wmbsFilesetFilesTable = Tbl(TABLES.WMBS_FILESET_FILES, True)
                wmbsFilesetTable = Tbl(TABLES.WMBS_FILESET, True)
                processedDatasetTable = Tbl(TABLES.PROCESSED_DATASET, True)

		self.selectCond = """, pds.name as skim"""

		self.whereCond = """ and m.run = """ + str(runid) + """ and fs.name = 'DBSUploadable' """

                self.fromCond = """ INNER JOIN """ + wmbsFileRunlumiMapTable + """ m ON (f.id = m.fileid)
                                    INNER JOIN """ + wmbsFilesetFilesTable + """ fsf ON (f.id = fsf.fileid)
                                    INNER JOIN """ + wmbsFilesetTable + """ fs ON (fs.id = fsf.fileset) 
				    INNER JOIN """ + processedDatasetTable + """ pds ON (dp.processed_dataset = pds.id) """

                BaseAlcaskimDP._load(self)


class AlcaSkimByIdsDP(BaseAlcaskimDP):
        def __init__(self, ids, orderby = 0, asc = True, page = 0):
                BaseAlcaskimDP.__init__(self, orderby, asc, page)
                self.title = "Temporary AlcaSkim files"

	        wmbsFilesetFilesTable = Tbl(TABLES.WMBS_FILESET_FILES, True)
        	wmbsFilesetTable = Tbl(TABLES.WMBS_FILESET, True)
                processedDatasetTable = Tbl(TABLES.PROCESSED_DATASET, True)
	
	        ids_str = ""
	        numIds = len(ids)
	        if numIds != 0:
	                if numIds > 1:
	                        for i in range(numIds-1):
	                                ids_str += str(ids[i]) + ", "
	                ids_str += str(ids[-1])

		self.selectCond = """, pds.name as skim"""
	
	        self.whereCond = """ and fs.name = 'Mergeable' and f.id IN (""" + ids_str + """) """
	
                self.fromCond = """ INNER JOIN """ + wmbsFilesetFilesTable + """ fsf ON (f.id = fsf.fileid)
                                    INNER JOIN """ + wmbsFilesetTable + """ fs ON (fs.id = fsf.fileset)
                                    INNER JOIN """ + processedDatasetTable + """ pds ON (dp.processed_dataset = pds.id) """
	
	        BaseAlcaskimDP._load(self)



class AlcaSkimByRecoMergeIdsDp(BaseAlcaskimDP):
        def __init__(self, ids, orderby = 0, asc = True, page = 0):
                BaseAlcaskimDP.__init__(self, orderby, asc, page)
                self.title = "Temporary AlcaSkim files"

	        wmbsFilesetFilesTable = Tbl(TABLES.WMBS_FILESET_FILES, True)
	        wmbsFilesetTable = Tbl(TABLES.WMBS_FILESET, True)
	        wmbsFileParentTable = Tbl(TABLES.WMBS_FILE_PARENT, True)
                processedDatasetTable = Tbl(TABLES.PROCESSED_DATASET, True)

	        ids_str = ""
	        numIds = len(ids)
	        if numIds != 0:
	                if numIds > 1:
	                        for i in range(numIds-1):
	                                ids_str += str(ids[i]) + ", "
	                ids_str += str(ids[-1])

                self.selectCond = """, pds.name as skim"""

	        self.whereCond = """ and fs.name = 'Mergeable' and fp.parent IN (""" + ids_str + """) """
	
	        self.fromCond = """ INNER JOIN """ + wmbsFilesetFilesTable + """ fsf ON (f.id = fsf.fileid)
	                            INNER JOIN """ + wmbsFilesetTable + """ fs ON (fs.id = fsf.fileset)
	                            INNER JOIN """ + wmbsFileParentTable + """ fp ON (f.id = fp.child) 
				    INNER JOIN """ + processedDatasetTable + """ pds ON (dp.processed_dataset = pds.id) """
	
	        if len(ids) != 0:
	        	BaseAlcaskimDP._load(self)


class AlcaSkimByAlcaSkimMergeJobIdsDP(BaseAlcaskimDP):
	def __init__(self, ids, orderby = 0, asc = True, page = 0):
        	BaseAlcaskimDP.__init__(self, orderby, asc, page)
	        self.title = "Temporary AlcaSkim files"
	
	        wmbsJobAssocTable = Tbl(TABLES.WMBS_JOB_ASSOC, True)
	        wmbsFilesetFilesTable = Tbl(TABLES.WMBS_FILESET_FILES, True)
	        wmbsFilesetTable = Tbl(TABLES.WMBS_FILESET, True)
                processedDatasetTable = Tbl(TABLES.PROCESSED_DATASET, True)

	        ids_str = ""
	        numIds = len(ids)
	        if numIds != 0:
	                if numIds > 1:
	                        for i in range(numIds-1):
	                                ids_str += str(ids[i]) + ", "
	                ids_str += str(ids[-1])

                self.selectCond = """, pds.name as skim"""
	
	        self.whereCond = """ and fs.name = 'Mergeable' and ja.job IN (""" + ids_str + """) """
	
	        self.fromCond = """ INNER JOIN """ + wmbsFilesetFilesTable + """ fsf ON (f.id = fsf.fileid)
	                            INNER JOIN """ + wmbsFilesetTable + """ fs ON (fs.id = fsf.fileset)
	                            INNER JOIN """ + wmbsJobAssocTable + """ ja ON (ja.fileid = f.id) 
				    INNER JOIN """ + processedDatasetTable + """ pds ON (dp.processed_dataset = pds.id) """
	
	        if len(ids) != 0:
	        	BaseAlcaskimDP._load(self)


class AlcaSkimMergeByAlcaSkimIdsDP(BaseAlcaskimDP):
        def __init__(self, ids, orderby = 0, asc = True, page = 0):
                BaseAlcaskimDP.__init__(self, orderby, asc, page)
		self.cols["Id"].decorator = buildAlcaSkimMergeLink

                wmbsFilesetFilesTable = Tbl(TABLES.WMBS_FILESET_FILES, True)
                wmbsFilesetTable = Tbl(TABLES.WMBS_FILESET, True)
                wmbsFileParentTable = Tbl(TABLES.WMBS_FILE_PARENT, True)
                processedDatasetTable = Tbl(TABLES.PROCESSED_DATASET, True)

                ids_str = ""
                numIds = len(ids)
                if numIds != 0:
                        if numIds > 1:
                                for i in range(numIds-1):
                                        ids_str += str(ids[i]) + ", "
                        ids_str += str(ids[-1])

                self.selectCond = """, pds.name as skim"""

                self.whereCond = """ and fs.name = 'DBSUploadable' and fp.parent IN (""" + ids_str + """) """

                self.fromCond = """ INNER JOIN """ + wmbsFilesetFilesTable + """ fsf ON (f.id = fsf.fileid)
                                    INNER JOIN """ + wmbsFilesetTable + """ fs ON (fs.id = fsf.fileset)
                                    INNER JOIN """ + wmbsFileParentTable + """ fp ON (f.id = fp.child)
                                    INNER JOIN """ + processedDatasetTable + """ pds ON (dp.processed_dataset = pds.id) """

                if len(ids) != 0:
                        BaseAlcaskimDP._load(self)


class AlcaSkimMergeByIdsDP(BaseAlcaskimDP):
        def __init__(self, ids, orderby = 0, asc = True, page = 0):
                BaseAlcaskimDP.__init__(self, orderby, asc, page)
		self.cols["Id"].decorator = buildAlcaSkimMergeLink

                wmbsFilesetFilesTable = Tbl(TABLES.WMBS_FILESET_FILES, True)
                wmbsFilesetTable = Tbl(TABLES.WMBS_FILESET, True)
                processedDatasetTable = Tbl(TABLES.PROCESSED_DATASET, True)

                ids_str = ""
                numIds = len(ids)
                if numIds != 0:
                        if numIds > 1:
                                for i in range(numIds-1):
                                        ids_str += str(ids[i]) + ", "
                        ids_str += str(ids[-1])

                self.selectCond = """, pds.name as skim"""

                self.whereCond = """ and fs.name = 'DBSUploadable' and f.id IN (""" + ids_str + """) """

                self.fromCond = """ INNER JOIN """ + wmbsFilesetFilesTable + """ fsf ON (f.id = fsf.fileid)
                                    INNER JOIN """ + wmbsFilesetTable + """ fs ON (fs.id = fsf.fileset)
                                    INNER JOIN """ + processedDatasetTable + """ pds ON (dp.processed_dataset = pds.id) """

                BaseAlcaskimDP._load(self)


class AlcaSkimByAlcaSkimMergeIdsDP(BaseAlcaskimDP):
    def __init__(self, ids, orderby = 0, asc = True, page = 0):
        BaseAlcaskimDP.__init__(self, orderby, asc, page)
        self.title = "Temporary AlcaSkim files"

        wmbsFilesetFilesTable = Tbl(TABLES.WMBS_FILESET_FILES, True)
        wmbsFilesetTable = Tbl(TABLES.WMBS_FILESET, True)
        wmbsFileParentTable = Tbl(TABLES.WMBS_FILE_PARENT, True)
	processedDatasetTable = Tbl(TABLES.PROCESSED_DATASET, True)

        ids_str = ""
        numIds = len(ids)
        if numIds != 0:
                if numIds > 1:
                        for i in range(numIds-1):
                                ids_str += str(ids[i]) + ", "
                ids_str += str(ids[-1])

	self.selectCond = """, pds.name as skim"""

        self.whereCond = """ and fs.name = 'Mergeable' and fp.child IN (""" + ids_str + """) """

        self.fromCond = """ INNER JOIN """ + wmbsFilesetFilesTable + """ fsf ON (f.id = fsf.fileid)
                            INNER JOIN """ + wmbsFilesetTable + """ fs ON (fs.id = fsf.fileset)
                            INNER JOIN """ + wmbsFileParentTable + """ fp ON (f.id = fp.parent) 
			    INNER JOIN """ + processedDatasetTable + """ pds ON (dp.processed_dataset = pds.id) """

        if len(ids) != 0:
            BaseAlcaskimDP._load(self)


class RecoByIdsDP(BaseRecoDP):
    def __init__(self, ids, orderby = 0, asc = True, page = 0):
        BaseRecoDP.__init__(self, orderby, asc, page)
        self.title = "Temporary Reco files"

        wmbsFilesetFilesTable = Tbl(TABLES.WMBS_FILESET_FILES, True)
        wmbsFilesetTable = Tbl(TABLES.WMBS_FILESET, True)

        ids_str = ""
        numIds = len(ids)
        if numIds != 0:
                if numIds > 1:
                        for i in range(numIds-1):
                                ids_str += str(ids[i]) + ", "
                ids_str += str(ids[-1])

        self.whereCond = """ and fs.name = 'Mergeable' and f.id IN (""" + ids_str + """) """

        self.fromCond = """ INNER JOIN """ + wmbsFilesetFilesTable + """ fsf ON (f.id = fsf.fileid)
                            INNER JOIN """ + wmbsFilesetTable + """ fs ON (fs.id = fsf.fileset) """

        BaseRecoDP._load(self)

class RecoByRunIdDP(BaseRecoDP):
    def __init__(self, runid, orderby = 0, asc = True, page = 0):
        BaseRecoDP.__init__(self, orderby, asc, page)
        self.title = "Temporary Reco files"

        wmbsFileRunlumiMapTable = Tbl(TABLES.WMBS_FILE_RUNLUMI_MAP, True)
        wmbsFilesetFilesTable = Tbl(TABLES.WMBS_FILESET_FILES, True)
        wmbsFilesetTable = Tbl(TABLES.WMBS_FILESET, True)

        self.whereCond = """ and m.run = """ + str(runid) + """ and fs.name = 'Mergeable' """

        self.fromCond = """ INNER JOIN """ + wmbsFileRunlumiMapTable + """ m ON (f.id = m.fileid)
                            INNER JOIN """ + wmbsFilesetFilesTable + """ fsf ON (f.id = fsf.fileid)
                            INNER JOIN """ + wmbsFilesetTable + """ fs ON (fs.id = fsf.fileset) """

        BaseRecoDP._load(self)

class RecoByRecoMergeJobIdsDP(BaseRecoDP):
    def __init__(self, ids, orderby = 0, asc = True, page = 0):
        BaseRecoDP.__init__(self, orderby, asc, page)
        self.title = "Temporary Reco files"

        wmbsJobAssocTable = Tbl(TABLES.WMBS_JOB_ASSOC, True)
        wmbsFilesetFilesTable = Tbl(TABLES.WMBS_FILESET_FILES, True)
        wmbsFilesetTable = Tbl(TABLES.WMBS_FILESET, True)

        ids_str = ""
        numIds = len(ids)
        if numIds != 0:
                if numIds > 1:
                        for i in range(numIds-1):
                                ids_str += str(ids[i]) + ", "
                ids_str += str(ids[-1])

        self.whereCond = """ and fs.name = 'Mergeable' and ja.job IN (""" + ids_str + """) """

        self.fromCond = """ INNER JOIN """ + wmbsFilesetFilesTable + """ fsf ON (f.id = fsf.fileid)
                            INNER JOIN """ + wmbsFilesetTable + """ fs ON (fs.id = fsf.fileset)
			    INNER JOIN """ + wmbsJobAssocTable + """ ja ON (ja.fileid = f.id) """

        if len(ids) != 0:
            BaseRecoDP._load(self)


class RecoByMergeIdsDP(BaseRecoDP):
    def __init__(self, ids, orderby = 0, asc = True, page = 0):
        BaseRecoDP.__init__(self, orderby, asc, page)
        self.title = "Temporary Reco files"

        wmbsFilesetFilesTable = Tbl(TABLES.WMBS_FILESET_FILES, True)
        wmbsFilesetTable = Tbl(TABLES.WMBS_FILESET, True)
        wmbsFileParentTable = Tbl(TABLES.WMBS_FILE_PARENT, True)

        ids_str = ""
        numIds = len(ids)
        if numIds != 0:
                if numIds > 1:
                        for i in range(numIds-1):
                                ids_str += str(ids[i]) + ", "
                ids_str += str(ids[-1])

        self.whereCond = """ and fs.name = 'Mergeable' and fp.parent IN (""" + ids_str + """) """

        self.fromCond = """ INNER JOIN """ + wmbsFilesetFilesTable + """ fsf ON (f.id = fsf.fileid)
                            INNER JOIN """ + wmbsFilesetTable + """ fs ON (fs.id = fsf.fileset)
                            INNER JOIN """ + wmbsFileParentTable + """ fp ON (f.id = fp.child) """

        if len(ids) != 0:
            BaseRecoDP._load(self)

class RecoByRecoMergeIdsDP(BaseRecoDP):
    def __init__(self, ids, orderby = 0, asc = True, page = 0):
        BaseRecoDP.__init__(self, orderby, asc, page)
        self.title = "Temporary Reco files"

        wmbsFilesetFilesTable = Tbl(TABLES.WMBS_FILESET_FILES, True)
        wmbsFilesetTable = Tbl(TABLES.WMBS_FILESET, True)
        wmbsFileParentTable = Tbl(TABLES.WMBS_FILE_PARENT, True)

        ids_str = ""
        numIds = len(ids)
        if numIds != 0:
                if numIds > 1:
                        for i in range(numIds-1):
                                ids_str += str(ids[i]) + ", "
                ids_str += str(ids[-1])

        self.whereCond = """ and fs.name = 'Mergeable' and fp.child IN (""" + ids_str + """) """

        self.fromCond = """ INNER JOIN """ + wmbsFilesetFilesTable + """ fsf ON (f.id = fsf.fileid)
                            INNER JOIN """ + wmbsFilesetTable + """ fs ON (fs.id = fsf.fileset)
                            INNER JOIN """ + wmbsFileParentTable + """ fp ON (f.id = fp.parent) """

        if len(ids) != 0:
            BaseRecoDP._load(self)


class StreamerDP(PagedTableDataProvider):
    cols = Cols([
        Col("Id", decorator = buildStreamerLink),
        Col("Lumi Id"),
        Col("Filesize", decorator = humanFilesize),
        Col("Events"),
        Col("Name"),
        Col("E"),   # TODO : check if this column is still necessary, it doesn't appear in streamer table
#        Col("S"),
        Col("Lfn", decorator = lambda x: textToImageAlt(x))
    ])

    def __init__(self, orderby = 0, asc = True, page = 0):
        PagedTableDataProvider.__init__(self, orderby, asc, page)
        self.title = 'Streamer files'

    def _buildCountQuery(self):
        streamerTable = Tbl(TABLES.STREAMER)

        self.countQuery = select(
            [func.count(streamerTable.c.streamer_id)],
            self.whereCond
        )

    def _buildMainQuery(self):
        streamerTable = Tbl(TABLES.STREAMER)
        streamTable = Tbl(TABLES.STREAM)

        c = (
            streamerTable.c.streamer_id,
            streamerTable.c.lumi_id,
            streamerTable.c.filesize,
            streamerTable.c.events,
            streamTable.c.name,
            streamerTable.c.exportable,
#            streamerTable.c.splitable,
            streamerTable.c.lfn
        )

        if self.asc: o = c[self.orderby].asc()
        else: o = c[self.orderby].desc()

        self.mainQuery = select(
            c,
            self.whereCond,
	    from_obj = [streamerTable.outerjoin(streamTable, streamerTable.c.stream_id == streamTable.c.id)]
        ).order_by(o)

class StreamerByRunIdDP(StreamerDP):
    def __init__(self, runid, orderby = 0, asc = True, page = 0):
        StreamerDP.__init__(self, orderby, asc, page)
        streamerTable = Tbl(TABLES.STREAMER)
        self.whereCond = (streamerTable.c.run_id == runid)
        StreamerDP._load(self)

class StreamerByIdsDP(StreamerDP):
    def __init__(self, ids, orderby = 0, asc = True, page = 0):
        StreamerDP.__init__(self, orderby, asc, page)
        streamerTable = Tbl(TABLES.STREAMER)
        self.whereCond = (streamerTable.c.streamer_id.in_(ids))
        StreamerDP._load(self)

class StreamerByRepackJobIdsDP(StreamerDP):
    def __init__(self, ids, orderby = 0, asc = True, page = 0):
        StreamerDP.__init__(self, orderby, asc, page)
        streamerTable = Tbl(TABLES.STREAMER)
        assocTable = Tbl(TABLES.JOB_DATASET_STREAMER_ASSOC)
        self.whereCond = streamerTable.c.streamer_id.in_(
            select([func.distinct(assocTable.c.streamer_id)], assocTable.c.job_id.in_(ids))
        )
        StreamerDP._load(self)



